/*
 * Copyright 2013 Agustina Martinez-Garcia

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
 */
package uk.ljmu.fedoraifz.tests;

import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import org.dom4j.Document;
import org.dom4j.DocumentException;
import org.dom4j.DocumentHelper;
import org.igfay.jfig.JFig;
import org.igfay.jfig.JFigException;
import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.Test;

import uk.ljmu.fedoraifz.restrequests.FedoraOperationsAPIA;
import uk.ljmu.fedoraifz.restrequests.FedoraOperationsAPIAImpl;
import uk.ljmu.fedoraifz.restrequests.FedoraOperationsAPIM;
import uk.ljmu.fedoraifz.restrequests.FedoraOperationsAPIMImpl;

public class FedoraApiaTest {

    private static FedoraOperationsAPIA fedoraApia;
    private static String demoObject = "demo:test1";
    private static String user;
    private static String password;
    private static boolean abortAll = false;

    @BeforeClass
    public static void setUpBeforeClass() throws Exception {
	try {
	    JFig.initialize();
	    user = JFig.getInstance().getValue("fedoraServer", "user");
	    password = JFig.getInstance().getValue("fedoraServer", "password");
	} catch (JFigException e) {
	    throw new Exception("Test aborted errors while reading configuration file");
	}
	createObject();
	fedoraApia = new FedoraOperationsAPIAImpl();
    }

    @AfterClass
    public static void setUpAfterClass() throws Exception {
	if (!abortAll) {
	    FedoraOperationsAPIM fedoraObj = new FedoraOperationsAPIMImpl(user, password);
	    String results = fedoraObj.getObjectXML(demoObject);
	    if (!results.equalsIgnoreCase(demoObject))
		deleteObject();
	}
    }

    private static void createObject() {
	//Check if object exists
	FedoraOperationsAPIM fedoraObj = new FedoraOperationsAPIMImpl("fedoraAdmin", "dynamic");
	String results = fedoraObj.getObjectXML(demoObject);
	if (results == null) {
	    //No connection or errors while connecting to Fedora
	    abortAll = true;
	} else {
	    if (results.trim().equals(demoObject))
		results = fedoraObj.createObject(demoObject);
	}
    }

    private static void deleteObject() {
	//Check if object exists
	FedoraOperationsAPIM fedoraObj = new FedoraOperationsAPIMImpl("fedoraAdmin", "dynamic");
	String results = fedoraObj.getObjectXML(demoObject);
	if (results.trim().equals(demoObject))
	    results = fedoraObj.deleteObject(demoObject);
	//System.out.println(results);
    }

    @Test
    public void testFindObjectsOk() {
	if (!abortAll) {
	    //Search only the demo object, no query parameter, no maxResults, no sessionToken, resultFormat XML
	    String results = fedoraApia.findObjects(demoObject, "", null, "", "xml");
	    try {
		//XML
		Document document = DocumentHelper.parseText(results);
		assertTrue(document.hasContent());
	    } catch (DocumentException e) {
		fail(e.getMessage());
	    }

	    //Search without parameters
	    results = fedoraApia.findObjects("", "", null, "", "xml");
	    try {
		//XML
		Document document = DocumentHelper.parseText(results);
		assertTrue(document.hasContent());
	    } catch (DocumentException e) {
		fail(e.getMessage());
	    }
	} else fail("No connection to Fedora");
    }
    @Test
    public void testNoConnection() {
	if (abortAll)
	    assert(true);
    }

    @Test
    public void testListDatastreamsOk() {
	if (!abortAll) {
	    String results = fedoraApia.listDatastreams(demoObject, "xml", "");
	    try {
		//XML
		Document document = DocumentHelper.parseText(results);
		assertTrue(document.hasContent());
	    } catch (DocumentException e) {
		fail(e.getMessage());
	    }	
	} else fail("No connection to Fedora");
    }	

    @Test
    public void testListDatastreamsNoOk() {
	if (!abortAll) {
	    //No PID
	    String results = fedoraApia.listDatastreams("", "xml", "");
	    assertTrue(results == null);
	} else fail("No connection to Fedora");
    }

    @Test
    public void testGetObjectProfileOk() {
	if (!abortAll) {
	    String results = fedoraApia.getObjectProfile(demoObject, "xml", "");
	    try {
		//XML
		Document document = DocumentHelper.parseText(results);
		assertTrue(document.hasContent());
	    } catch (DocumentException e) {
		fail(e.getMessage());
	    }	
	} else fail("No connection to Fedora");
    }

    @Test
    public void testGetObjectProfileObjectNotFound() {
	if (!abortAll) {
	    String results = fedoraApia.getObjectProfile("fake:object", "xml", "");
	    assertTrue(results == null);
	} else fail("No connection to Fedora");
    }
    
    @Test
    public void testGetObjectProfileNoPID() {
	if (!abortAll) {
	    String results = fedoraApia.getObjectProfile("", "xml", "");
	    assertTrue(results == null);
	} else fail("No connection to Fedora");
    }
}
