/*
 * Copyright 2013 Agustina Martinez-Garcia

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
 */
package uk.ljmu.fedoraifz.tests;

import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import org.dom4j.Document;
import org.dom4j.DocumentException;
import org.dom4j.DocumentHelper;
import org.igfay.jfig.JFig;
import org.igfay.jfig.JFigException;
import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.Ignore;
import org.junit.Test;

import uk.ljmu.fedoraifz.restrequests.FedoraOperationsAPIM;
import uk.ljmu.fedoraifz.restrequests.FedoraOperationsAPIMImpl;

public class FedoraApimTest {
    private static FedoraOperationsAPIM fedoraApim;
    private static boolean abortAll = false;
    private static String demoObject = "demo:test1";
    private static String user;
    private static String password;

    @BeforeClass
    public static void setUpBeforeClass() throws Exception {
	try {
	    JFig.initialize();   
	    user = JFig.getInstance().getValue("fedoraServer", "user");
	    password = JFig.getInstance().getValue("fedoraServer", "password");
	} catch (JFigException e) {
	    throw new Exception("Test aborted errors while reading configuration file");
	}
	fedoraApim = new FedoraOperationsAPIMImpl();
	
	createObject();
    }
    
    @AfterClass
    public static void setUpAfterClass() throws Exception {
	if (!abortAll) {
	    FedoraOperationsAPIM fedoraObj = new FedoraOperationsAPIMImpl(user, password);
	    String results = fedoraObj.getObjectXML(demoObject);
	    if (!results.equalsIgnoreCase(demoObject))
		deleteObject();
	}
    }

    private static void createObject() {
	//Check if object exists
	FedoraOperationsAPIM fedoraObj = new FedoraOperationsAPIMImpl("fedoraAdmin", "dynamic");
	String results = fedoraObj.getObjectXML(demoObject);
	if (results == null) {
	    //No connection or errors while connecting to Fedora
	    abortAll = true;
	} else {
	    if (results.trim().equals(demoObject))
		results = fedoraObj.createObject(demoObject);
	}
    }

    private static void deleteObject() {
	//Check if object exists
	FedoraOperationsAPIM fedoraObj = new FedoraOperationsAPIMImpl("fedoraAdmin", "dynamic");
	String results = fedoraObj.getObjectXML(demoObject);
	if (results.trim().equals(demoObject))
	    results = fedoraObj.deleteObject(demoObject);
	System.out.println(results);
    }
    @Test
    public void testGetNextPID() {
	if (!abortAll) {
	    String results = fedoraApim.getNextPID();
	    try {
		Document doc = DocumentHelper.parseText(results);
		assertTrue(doc.hasContent());
	    } catch (DocumentException e) {
		fail("getNextPid invalid results");
	    }
	}
	else fail("Fedora instance not running, connection error");
    }
    
    @Test
    public void testGetNextPIDParameters() {
	if (!abortAll) {
	    String results = fedoraApim.getNextPID(3, "", "xml");
	    try {
		Document doc = DocumentHelper.parseText(results);
		assertTrue(doc.hasContent());
	    } catch (DocumentException e) {
		fail("getNextPidParameters invalid results");
	    }
	}
	else fail("Fedora instance not running, connection error");
    }
    @Test
    public void testAddDatastream() {
	if (!abortAll) {

	}
	else fail("Fedora instance not running, connection error");
    }
    @Test
    public void testGetDatastream() {
	if (!abortAll) {

	}
	else fail("Fedora instance not running, connection error");
    }
    @Test
    public void testCreateObject() {
	if (!abortAll) {

	}
	else fail("Fedora instance not running, connection error");
    }
    @Test
    public void tesDeleteObject() {
	if (!abortAll) {

	}
	else fail("Fedora instance not running, connection error");
    }
    @Test
    public void testDeleteDatastream() {
	if (!abortAll) {

	}
	else fail("Fedora instance not running, connection error");
    }
    @Test
    public void testGetObjectXML() {
	if (!abortAll) {

	}
	else fail("Fedora instance not running, connection error");
    }
    
    @Test
    public void testGetDatastreamControlGroup() {
	if (!abortAll) {

	}
	else fail("Fedora instance not running, connection error");
    }
    
    @Test
    public void testValidate() {
	if (!abortAll) {
	    String results = fedoraApim.validate(demoObject, "");
	    try {
		Document doc = DocumentHelper.parseText(results);
		assertTrue(doc.hasContent());
	    } catch (DocumentException e) {
		fail("testValidate invalid results");
	    }
	}
	else fail("Fedora instance not running, connection error");
    }
    @Test
    public void testValidateNoPID() {
	if (!abortAll) {
	    String results = fedoraApim.validate("", "");
	    if (results == null)
		assert(true);
	    else fail("testValidate invalid results");
	}
	else fail("Fedora instance not running, connection error");
    }
    @Ignore
    public void testAddRelationship() {
	if (!abortAll) {

	}
	else fail("Fedora instance not running, connection error");
    }
    @Ignore
    public void testGetRelationships() {
	if (!abortAll) {

	}
	else fail("Fedora instance not running, connection error");
    }
    @Ignore
    public void testPurgeRelationship() {
	if (!abortAll) {

	}
	else fail("Fedora instance not running, connection error");
    }
}
