/*
 * Copyright 2013 Agustina Martinez-Garcia

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
 */
package uk.ljmu.metadata.staticschemas;

import java.io.InputStream;

import com.hp.hpl.jena.rdf.model.Model;
import com.hp.hpl.jena.rdf.model.ModelFactory;
import com.hp.hpl.jena.rdf.model.Property;
import com.hp.hpl.jena.rdf.model.Resource;

public class StaticJenaFoaf {
	 /** <p>The RDF model that holds the vocabulary terms</p> */
    private static Model m_model = ModelFactory.createDefaultModel();
    
    /** <p>The namespace of the vocabulary as a string</p> */
    public static final String NS = "http://xmlns.com/foaf/0.1/";
    
    /** <p>The namespace of the vocabulary as a string</p>
     *  @see #NS */
    public static String getURI() {return NS;}
    
    /** <p>The namespace of the vocabulary as a resource</p> */
    public static final Resource NAMESPACE = m_model.createResource( NS );
    
    /** <p>Indicates an account held by this agent.</p> */
    public static final Property account = m_model.createProperty( "http://xmlns.com/foaf/0.1/account" );
    
    /** <p>Indicates the name (identifier) associated with this online account.</p> */
    public static final Property accountName = m_model.createProperty( "http://xmlns.com/foaf/0.1/accountName" );
    
    /** <p>Indicates a homepage of the service provide for this online account.</p> */
    public static final Property accountServiceHomepage = m_model.createProperty( "http://xmlns.com/foaf/0.1/accountServiceHomepage" );
    
    /** <p>The age in years of some agent.</p> */
    public static final Property age = m_model.createProperty( "http://xmlns.com/foaf/0.1/age" );
    
    /** <p>An AIM chat ID</p> */
    public static final Property aimChatID = m_model.createProperty( "http://xmlns.com/foaf/0.1/aimChatID" );
    
    /** <p>A location that something is based near, for some broadly human notion of 
     *  near.</p>
     */
    public static final Property based_near = m_model.createProperty( "http://xmlns.com/foaf/0.1/based_near" );
    
    /** <p>The birthday of this Agent, represented in mm-dd string form, eg. '12-31'.</p> */
    public static final Property birthday = m_model.createProperty( "http://xmlns.com/foaf/0.1/birthday" );
    
    /** <p>A current project this person works on.</p> */
    public static final Property currentProject = m_model.createProperty( "http://xmlns.com/foaf/0.1/currentProject" );
    
    /** <p>A depiction of some thing.</p> */
    public static final Property depiction = m_model.createProperty( "http://xmlns.com/foaf/0.1/depiction" );
    
    /** <p>A thing depicted in this representation.</p> */
    public static final Property depicts = m_model.createProperty( "http://xmlns.com/foaf/0.1/depicts" );
    
    /** <p>A checksum for the DNA of some thing. Joke.</p> */
    public static final Property dnaChecksum = m_model.createProperty( "http://xmlns.com/foaf/0.1/dnaChecksum" );
    
    /** <p>The family name of some person.</p> */
    public static final Property familyName = m_model.createProperty( "http://xmlns.com/foaf/0.1/familyName" );
    
    /** <p>The family name of some person.</p> */
    public static final Property family_name = m_model.createProperty( "http://xmlns.com/foaf/0.1/family_name" );
    
    /** <p>The first name of a person.</p> */
    public static final Property firstName = m_model.createProperty( "http://xmlns.com/foaf/0.1/firstName" );
    
    /** <p>An organization funding a project or person.</p> */
    public static final Property fundedBy = m_model.createProperty( "http://xmlns.com/foaf/0.1/fundedBy" );
    
    /** <p>A textual geekcode for this person, see http://www.geekcode.com/geek.html</p> */
    public static final Property geekcode = m_model.createProperty( "http://xmlns.com/foaf/0.1/geekcode" );
    
    /** <p>The gender of this Agent (typically but not necessarily 'male' or 'female').</p> */
    public static final Property gender = m_model.createProperty( "http://xmlns.com/foaf/0.1/gender" );
    
    /** <p>The given name of some person.</p> */
    public static final Property givenName = m_model.createProperty( "http://xmlns.com/foaf/0.1/givenName" );
    
    /** <p>The given name of some person.</p> */
    public static final Property givenname = m_model.createProperty( "http://xmlns.com/foaf/0.1/givenname" );
    
    /** <p>Indicates an account held by this agent.</p> */
    public static final Property holdsAccount = m_model.createProperty( "http://xmlns.com/foaf/0.1/holdsAccount" );
    
    /** <p>A homepage for some thing.</p> */
    public static final Property homepage = m_model.createProperty( "http://xmlns.com/foaf/0.1/homepage" );
    
    /** <p>An ICQ chat ID</p> */
    public static final Property icqChatID = m_model.createProperty( "http://xmlns.com/foaf/0.1/icqChatID" );
    
    /** <p>An image that can be used to represent some thing (ie. those depictions which 
     *  are particularly representative of something, eg. one's photo on a homepage).</p>
     */
    public static final Property img = m_model.createProperty( "http://xmlns.com/foaf/0.1/img" );
    
    /** <p>A page about a topic of interest to this person.</p> */
    public static final Property interest = m_model.createProperty( "http://xmlns.com/foaf/0.1/interest" );
    
    /** <p>A document that this thing is the primary topic of.</p> */
    public static final Property isPrimaryTopicOf = m_model.createProperty( "http://xmlns.com/foaf/0.1/isPrimaryTopicOf" );
    
    /** <p>A jabber ID for something.</p> */
    public static final Property jabberID = m_model.createProperty( "http://xmlns.com/foaf/0.1/jabberID" );
    
    /** <p>A person known by this person (indicating some level of reciprocated interaction 
     *  between the parties).</p>
     */
    public static final Property knows = m_model.createProperty( "http://xmlns.com/foaf/0.1/knows" );
    
    /** <p>The last name of a person.</p> */
    public static final Property lastName = m_model.createProperty( "http://xmlns.com/foaf/0.1/lastName" );
    
    /** <p>A logo representing some thing.</p> */
    public static final Property logo = m_model.createProperty( "http://xmlns.com/foaf/0.1/logo" );
    
    /** <p>Something that was made by this agent.</p> */
    public static final Property made = m_model.createProperty( "http://xmlns.com/foaf/0.1/made" );
    
    /** <p>An agent that made this thing.</p> */
    public static final Property maker = m_model.createProperty( "http://xmlns.com/foaf/0.1/maker" );
    
    /** <p>A personal mailbox, ie. an Internet mailbox associated with exactly one owner, 
     *  the first owner of this mailbox. This is a 'static inverse functional property', 
     *  in that there is (across time and change) at most one individual that ever 
     *  has any particular value for foaf:mbox.</p>
     */
    public static final Property mbox = m_model.createProperty( "http://xmlns.com/foaf/0.1/mbox" );
    
    /** <p>The sha1sum of the URI of an Internet mailbox associated with exactly one 
     *  owner, the first owner of the mailbox.</p>
     */
    public static final Property mbox_sha1sum = m_model.createProperty( "http://xmlns.com/foaf/0.1/mbox_sha1sum" );
    
    /** <p>Indicates a member of a Group</p> */
    public static final Property member = m_model.createProperty( "http://xmlns.com/foaf/0.1/member" );
    
    /** <p>Indicates the class of individuals that are a member of a Group</p> */
    public static final Property membershipClass = m_model.createProperty( "http://xmlns.com/foaf/0.1/membershipClass" );
    
    /** <p>An MSN chat ID</p> */
    public static final Property msnChatID = m_model.createProperty( "http://xmlns.com/foaf/0.1/msnChatID" );
    
    /** <p>A Myers Briggs (MBTI) personality classification.</p> */
    public static final Property myersBriggs = m_model.createProperty( "http://xmlns.com/foaf/0.1/myersBriggs" );
    
    /** <p>A name for some thing.</p> */
    public static final Property name = m_model.createProperty( "http://xmlns.com/foaf/0.1/name" );
    
    /** <p>A short informal nickname characterising an agent (includes login identifiers, 
     *  IRC and other chat nicknames).</p>
     */
    public static final Property nick = m_model.createProperty( "http://xmlns.com/foaf/0.1/nick" );
    
    /** <p>An OpenID for an Agent.</p> */
    public static final Property openid = m_model.createProperty( "http://xmlns.com/foaf/0.1/openid" );
    
    /** <p>A page or document about this thing.</p> */
    public static final Property page = m_model.createProperty( "http://xmlns.com/foaf/0.1/page" );
    
    /** <p>A project this person has previously worked on.</p> */
    public static final Property pastProject = m_model.createProperty( "http://xmlns.com/foaf/0.1/pastProject" );
    
    /** <p>A phone, specified using fully qualified tel: URI scheme (refs: http://www.w3.org/Addressing/schemes.html#tel).</p> */
    public static final Property phone = m_model.createProperty( "http://xmlns.com/foaf/0.1/phone" );
    
    /** <p>A .plan comment, in the tradition of finger and '.plan' files.</p> */
    public static final Property plan = m_model.createProperty( "http://xmlns.com/foaf/0.1/plan" );
    
    /** <p>The primary topic of some page or document.</p> */
    public static final Property primaryTopic = m_model.createProperty( "http://xmlns.com/foaf/0.1/primaryTopic" );
    
    /** <p>A link to the publications of this person.</p> */
    public static final Property publications = m_model.createProperty( "http://xmlns.com/foaf/0.1/publications" );
    
    /** <p>A homepage of a school attended by the person.</p> */
    public static final Property schoolHomepage = m_model.createProperty( "http://xmlns.com/foaf/0.1/schoolHomepage" );
    
    /** <p>A sha1sum hash, in hex.</p> */
    public static final Property sha1 = m_model.createProperty( "http://xmlns.com/foaf/0.1/sha1" );
    
    /** <p>A Skype ID</p> */
    public static final Property skypeID = m_model.createProperty( "http://xmlns.com/foaf/0.1/skypeID" );
    
    /** <p>A string expressing what the user is happy for the general public (normally) 
     *  to know about their current activity.</p>
     */
    public static final Property status = m_model.createProperty( "http://xmlns.com/foaf/0.1/status" );
    
    /** <p>The surname of some person.</p> */
    public static final Property surname = m_model.createProperty( "http://xmlns.com/foaf/0.1/surname" );
    
    /** <p>A theme.</p> */
    public static final Property theme = m_model.createProperty( "http://xmlns.com/foaf/0.1/theme" );
    
    /** <p>A derived thumbnail image.</p> */
    public static final Property thumbnail = m_model.createProperty( "http://xmlns.com/foaf/0.1/thumbnail" );
    
    /** <p>A tipjar document for this agent, describing means for payment and reward.</p> */
    public static final Property tipjar = m_model.createProperty( "http://xmlns.com/foaf/0.1/tipjar" );
    
    /** <p>Title (Mr, Mrs, Ms, Dr. etc)</p> */
    public static final Property title = m_model.createProperty( "http://xmlns.com/foaf/0.1/title" );
    
    /** <p>A topic of some page or document.</p> */
    public static final Property topic = m_model.createProperty( "http://xmlns.com/foaf/0.1/topic" );
    
    /** <p>A thing of interest to this person.</p> */
    public static final Property topic_interest = m_model.createProperty( "http://xmlns.com/foaf/0.1/topic_interest" );
    
    /** <p>A weblog of some thing (whether person, group, company etc.).</p> */
    public static final Property weblog = m_model.createProperty( "http://xmlns.com/foaf/0.1/weblog" );
    
    /** <p>A work info homepage of some person; a page about their work for some organization.</p> */
    public static final Property workInfoHomepage = m_model.createProperty( "http://xmlns.com/foaf/0.1/workInfoHomepage" );
    
    /** <p>A workplace homepage of some person; the homepage of an organization they 
     *  work for.</p>
     */
    public static final Property workplaceHomepage = m_model.createProperty( "http://xmlns.com/foaf/0.1/workplaceHomepage" );
    
    /** <p>A Yahoo chat ID</p> */
    public static final Property yahooChatID = m_model.createProperty( "http://xmlns.com/foaf/0.1/yahooChatID" );
    
    /** <p>An agent (eg. person, group, software or physical artifact).</p> */
    public static final Resource Agent = m_model.createResource( "http://xmlns.com/foaf/0.1/Agent" );
    
    /** <p>A document.</p> */
    public static final Resource Document = m_model.createResource( "http://xmlns.com/foaf/0.1/Document" );
    
    /** <p>A class of Agents.</p> */
    public static final Resource Group = m_model.createResource( "http://xmlns.com/foaf/0.1/Group" );
    
    /** <p>An image.</p> */
    public static final Resource Image = m_model.createResource( "http://xmlns.com/foaf/0.1/Image" );
    
    /** <p>A foaf:LabelProperty is any RDF property with texual values that serve as 
     *  labels.</p>
     */
    public static final Resource LabelProperty = m_model.createResource( "http://xmlns.com/foaf/0.1/LabelProperty" );
    
    /** <p>An online account.</p> */
    public static final Resource OnlineAccount = m_model.createResource( "http://xmlns.com/foaf/0.1/OnlineAccount" );
    
    /** <p>An online chat account.</p> */
    public static final Resource OnlineChatAccount = m_model.createResource( "http://xmlns.com/foaf/0.1/OnlineChatAccount" );
    
    /** <p>An online e-commerce account.</p> */
    public static final Resource OnlineEcommerceAccount = m_model.createResource( "http://xmlns.com/foaf/0.1/OnlineEcommerceAccount" );
    
    /** <p>An online gaming account.</p> */
    public static final Resource OnlineGamingAccount = m_model.createResource( "http://xmlns.com/foaf/0.1/OnlineGamingAccount" );
    
    /** <p>An organization.</p> */
    public static final Resource Organization = m_model.createResource( "http://xmlns.com/foaf/0.1/Organization" );
    
    /** <p>A person.</p> */
    public static final Resource Person = m_model.createResource( "http://xmlns.com/foaf/0.1/Person" );
    
    /** <p>A personal profile RDF document.</p> */
    public static final Resource PersonalProfileDocument = m_model.createResource( "http://xmlns.com/foaf/0.1/PersonalProfileDocument" );
    
    /** <p>A project (a collective endeavour of some kind).</p> */
    public static final Resource Project = m_model.createResource( "http://xmlns.com/foaf/0.1/Project" );
    /*public static Model getModel()
    {
    	Model myModel = ModelFactory.createDefaultModel();
    	myModel.createResource( NS );
    	myModel.createProperty( "http://xmlns.com/foaf/0.1/accountName" );
    	myModel.createProperty( "http://xmlns.com/foaf/0.1/accountServiceHomepage" );
    	myModel.createProperty( "http://xmlns.com/foaf/0.1/age" );
    	myModel.createProperty( "http://xmlns.com/foaf/0.1/aimChatID" );
        myModel.createProperty( "http://xmlns.com/foaf/0.1/based_near" );
        myModel.createProperty( "http://xmlns.com/foaf/0.1/birthday" );
        myModel.createProperty( "http://xmlns.com/foaf/0.1/currentProject" );
        myModel.createProperty( "http://xmlns.com/foaf/0.1/depiction" );
        myModel.createProperty( "http://xmlns.com/foaf/0.1/depicts" );
        myModel.createProperty( "http://xmlns.com/foaf/0.1/dnaChecksum" );
        myModel.createProperty( "http://xmlns.com/foaf/0.1/family_name" );
        myModel.createProperty( "http://xmlns.com/foaf/0.1/firstName" );
        myModel.createProperty( "http://xmlns.com/foaf/0.1/fundedBy" );
        myModel.createProperty( "http://xmlns.com/foaf/0.1/geekcode" );
        myModel.createProperty( "http://xmlns.com/foaf/0.1/gender" );
        myModel.createProperty( "http://xmlns.com/foaf/0.1/givenname" );
        myModel.createProperty( "http://xmlns.com/foaf/0.1/givenName" );
        myModel.createProperty( "http://xmlns.com/foaf/0.1/holdsAccount" );
        myModel.createProperty( "http://xmlns.com/foaf/0.1/homepage" );
        myModel.createProperty( "http://xmlns.com/foaf/0.1/icqChatID" );
        myModel.createProperty( "http://xmlns.com/foaf/0.1/img" );
        myModel.createProperty( "http://xmlns.com/foaf/0.1/interest" );
        myModel.createProperty( "http://xmlns.com/foaf/0.1/isPrimaryTopicOf" );
        myModel.createProperty( "http://xmlns.com/foaf/0.1/jabberID" );
        myModel.createProperty( "http://xmlns.com/foaf/0.1/knows" );
        myModel.createProperty( "http://xmlns.com/foaf/0.1/logo" );
        myModel.createProperty( "http://xmlns.com/foaf/0.1/made" );
        myModel.createProperty( "http://xmlns.com/foaf/0.1/maker" );
        myModel.createProperty( "http://xmlns.com/foaf/0.1/mbox" );
        myModel.createProperty( "http://xmlns.com/foaf/0.1/mbox_sha1sum" );
        myModel.createProperty( "http://xmlns.com/foaf/0.1/member" );
        myModel.createProperty( "http://xmlns.com/foaf/0.1/membershipClass" );
        myModel.createProperty( "http://xmlns.com/foaf/0.1/msnChatID" );
        myModel.createProperty( "http://xmlns.com/foaf/0.1/myersBriggs" );
        myModel.createProperty( "http://xmlns.com/foaf/0.1/name" );
        myModel.createProperty( "http://xmlns.com/foaf/0.1/nick" );
        myModel.createProperty( "http://xmlns.com/foaf/0.1/page" );
        myModel.createProperty( "http://xmlns.com/foaf/0.1/pastProject" );
        myModel.createProperty( "http://xmlns.com/foaf/0.1/phone" );
        myModel.createProperty( "http://xmlns.com/foaf/0.1/plan" );
        myModel.createProperty( "http://xmlns.com/foaf/0.1/primaryTopic" );
        myModel.createProperty( "http://xmlns.com/foaf/0.1/publications" );
        myModel.createProperty( "http://xmlns.com/foaf/0.1/schoolHomepage" );
        myModel.createProperty( "http://xmlns.com/foaf/0.1/sha1" );
        myModel.createProperty( "http://xmlns.com/foaf/0.1/surname" );
        myModel.createProperty( "http://xmlns.com/foaf/0.1/theme" );
        myModel.createProperty( "http://xmlns.com/foaf/0.1/thumbnail" );
        myModel.createProperty( "http://xmlns.com/foaf/0.1/tipjar" );
        myModel.createProperty( "http://xmlns.com/foaf/0.1/title" );
        myModel.createProperty( "http://xmlns.com/foaf/0.1/topic" );
        myModel.createProperty( "http://xmlns.com/foaf/0.1/topic_interest" );
        myModel.createProperty( "http://xmlns.com/foaf/0.1/weblog" );
        myModel.createProperty( "http://xmlns.com/foaf/0.1/workInfoHomepage" );
        myModel.createProperty( "http://xmlns.com/foaf/0.1/workplaceHomepage" );
        myModel.createProperty( "http://xmlns.com/foaf/0.1/yahooChatID" );
        
        return myModel;
    }*/
    
    public static Model getModel() {
	ClassLoader classLoader = Thread.currentThread()
	.getContextClassLoader();
	InputStream is;
	is = classLoader.getResourceAsStream("foaf.rdf");

    	Model myModel = ModelFactory.createDefaultModel();
    	
    	try {
    	    myModel.read(is, "");
    	    return myModel;
    	}catch (Exception ex) {
    	    return myModel;
    	}      
    }
    public static boolean containsProperty(String localName)
    {
    	if (StaticJenaFoaf.accountName.getLocalName().equals(localName) ||
    			StaticJenaFoaf.accountServiceHomepage.getLocalName().equals(localName) ||
    			StaticJenaFoaf.age.getLocalName().equals(localName) ||
    			StaticJenaFoaf.aimChatID.getLocalName().equals(localName) ||
    			StaticJenaFoaf.based_near.getLocalName().equals(localName) ||
    			StaticJenaFoaf.birthday.getLocalName().equals(localName) ||
    			StaticJenaFoaf.currentProject.getLocalName().equals(localName) ||
    			StaticJenaFoaf.depiction.getLocalName().equals(localName) ||
    			StaticJenaFoaf.depicts.getLocalName().equals(localName) ||
    			StaticJenaFoaf.dnaChecksum.getLocalName().equals(localName) ||
    			StaticJenaFoaf.family_name.getLocalName().equals(localName) ||
    			StaticJenaFoaf.firstName.getLocalName().equals(localName) ||
    			StaticJenaFoaf.fundedBy.getLocalName().equals(localName) ||
    			StaticJenaFoaf.geekcode.getLocalName().equals(localName) ||
    			StaticJenaFoaf.gender.getLocalName().equals(localName) ||
    			StaticJenaFoaf.givenname.getLocalName().equals(localName) ||
    			StaticJenaFoaf.givenName.getLocalName().equals(localName) ||
    			StaticJenaFoaf.holdsAccount.getLocalName().equals(localName) ||
    			StaticJenaFoaf.homepage.getLocalName().equals(localName) ||
    			StaticJenaFoaf.icqChatID.getLocalName().equals(localName) ||
    			StaticJenaFoaf.img.getLocalName().equals(localName) ||
    			StaticJenaFoaf.interest.getLocalName().equals(localName) ||
    			StaticJenaFoaf.isPrimaryTopicOf.getLocalName().equals(localName) ||
    			StaticJenaFoaf.jabberID.getLocalName().equals(localName) ||
    			StaticJenaFoaf.knows.getLocalName().equals(localName) ||
    			StaticJenaFoaf.logo.getLocalName().equals(localName) ||
    			StaticJenaFoaf.made.getLocalName().equals(localName) ||
    			StaticJenaFoaf.maker.getLocalName().equals(localName) ||
    			StaticJenaFoaf.mbox.getLocalName().equals(localName) ||
    			StaticJenaFoaf.mbox_sha1sum.getLocalName().equals(localName) ||
    			StaticJenaFoaf.member.getLocalName().equals(localName) ||
    			StaticJenaFoaf.membershipClass.getLocalName().equals(localName) ||
    			StaticJenaFoaf.msnChatID.getLocalName().equals(localName) ||
    			StaticJenaFoaf.myersBriggs.getLocalName().equals(localName) ||
    			StaticJenaFoaf.name.getLocalName().equals(localName) ||
    			StaticJenaFoaf.nick.getLocalName().equals(localName) ||
    			StaticJenaFoaf.page.getLocalName().equals(localName) ||
    			StaticJenaFoaf.pastProject.getLocalName().equals(localName) ||
    			StaticJenaFoaf.phone.getLocalName().equals(localName) ||
    			StaticJenaFoaf.plan.getLocalName().equals(localName) ||
    			StaticJenaFoaf.pastProject.getLocalName().equals(localName) ||
    			StaticJenaFoaf.primaryTopic.getLocalName().equals(localName) ||
    			StaticJenaFoaf.publications.getLocalName().equals(localName) ||
    			StaticJenaFoaf.schoolHomepage.getLocalName().equals(localName) ||
    			StaticJenaFoaf.sha1.getLocalName().equals(localName) ||
    			StaticJenaFoaf.surname.getLocalName().equals(localName) ||
    			StaticJenaFoaf.theme.getLocalName().equals(localName) ||
    			StaticJenaFoaf.thumbnail.getLocalName().equals(localName) ||
    			StaticJenaFoaf.tipjar.getLocalName().equals(localName) ||
    			StaticJenaFoaf.title.getLocalName().equals(localName) ||
    			StaticJenaFoaf.topic.getLocalName().equals(localName) ||
    			StaticJenaFoaf.topic_interest.getLocalName().equals(localName) ||
    			StaticJenaFoaf.weblog.getLocalName().equals(localName) ||
    			StaticJenaFoaf.workInfoHomepage.getLocalName().equals(localName) ||
    			StaticJenaFoaf.workplaceHomepage.getLocalName().equals(localName) ||
    			StaticJenaFoaf.yahooChatID.getLocalName().equals(localName))
    		return true;
    	else return false;
    }
    
    public static boolean containsResource(String localName)
    {
    	if (StaticJenaFoaf.Agent.getLocalName().equals(localName) ||
    			StaticJenaFoaf.Document.getLocalName().equals(localName) ||
    			StaticJenaFoaf.Group.getLocalName().equals(localName) ||
    			StaticJenaFoaf.Image.getLocalName().equals(localName) ||
    			StaticJenaFoaf.LabelProperty.getLocalName().equals(localName) ||
    			StaticJenaFoaf.OnlineAccount.getLocalName().equals(localName) ||
    			StaticJenaFoaf.OnlineChatAccount.getLocalName().equals(localName) ||
    			StaticJenaFoaf.OnlineEcommerceAccount.getLocalName().equals(localName) ||
    			StaticJenaFoaf.OnlineGamingAccount.getLocalName().equals(localName) ||
    			StaticJenaFoaf.Organization.getLocalName().equals(localName) ||
    			StaticJenaFoaf.Person.getLocalName().equals(localName) ||
    			StaticJenaFoaf.PersonalProfileDocument.getLocalName().equals(localName) ||
    			StaticJenaFoaf.Project.getLocalName().equals(localName))
    		return true;
    	else return false;
    }
}
