/*
 * Copyright 2013 Agustina Martinez-Garcia

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
 */
package uk.ljmu.mulgaraifz.tests;

import org.igfay.jfig.JFig;
//import org.igfay.jfig.JFigException;
import org.junit.BeforeClass;
import org.junit.Ignore;
import org.junit.Test;

import static org.junit.Assert.*;
import uk.ljmu.mulgaraifz.errorhandling.MulgaraException;
import uk.ljmu.mulgaraifz.httprequests.HttpManagement;

public class HttpManagementTest {
	private static HttpManagement httpMan;
	private static String testquery;
	private static String DEMO_RDF;
	static 
	{
		// Test FOXML object with RELS-EXT datastream
		StringBuilder sb = new StringBuilder();
		sb.append("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
		sb.append("<rdf:RDF xml:lang=\"en\"");
		sb.append("	   xmlns:rdf=\"http://www.w3.org/1999/02/22-rdf-syntax-ns#\"");
		sb.append("	   xmlns:rdfs=\"http://www.w3.org/2000/01/rdf-schema#\">");
		sb.append("			<!-- Note: this RDF schema would typically be used in RDF instance data ");
		sb.append("	     by referencing it with an XML namespace declaration, for example");
		sb.append("	     xmlns:xyz=\"http://www.w3.org/2000/03/example/vehicles#\".  This allows");
		sb.append("	     us to use abbreviations such as xyz:MotorVehicle to refer");
		sb.append("	     unambiguously to the RDF class 'MotorVehicle'. -->");
		sb.append("			<rdf:Description ID=\"MotorVehicle\">");
		sb.append("	 <rdf:type resource=\"http://www.w3.org/2000/01/rdf-schema#Class\"/>");
		sb.append("	 <rdfs:subClassOf");
		sb.append("	  rdf:resource=\"http://www.w3.org/2000/01/rdf-schema#Resource\"/>");
		sb.append("	</rdf:Description>");
		sb.append("			<rdf:Description ID=\"PassengerVehicle\">");
		sb.append("<rdf:type resource=\"http://www.w3.org/2000/01/rdf-schema#Class\"/>");
		sb.append("	 <rdfs:subClassOf rdf:resource=\"#MotorVehicle\"/>");
		sb.append("	</rdf:Description>");
		sb.append("			<rdf:Description ID=\"Truck\">");
		sb.append("	 <rdf:type resource=\"http://www.w3.org/2000/01/rdf-schema#Class\"/>");
		sb.append("	 <rdfs:subClassOf rdf:resource=\"#MotorVehicle\"/>");
		sb.append("	</rdf:Description>");
		sb.append("	</rdf:RDF>");

		try {
			DEMO_RDF = sb.toString();
		} catch (Exception uee) {
		}
	}
	private static byte[] rdfInvalidData;
	
	static {
		StringBuilder sb = new StringBuilder();
		
		sb.append("<?xml version=\"1.0\"?>");
		sb.append("<rdf:ROF ");
		sb.append("xmlns:rdf=\"http://www.w3.org/1999/02/22-rdf-syntax-ns#\" ");
		sb.append("xmlns:cd=\"http://www.recshop.fake/cd#\"> ");
		sb.append("<rdf:Description ");
		sb.append("rdf:about=\"http://www.recshop.fake/cd/EmpireBurlesque\" >");
		sb.append("<cd:artist>Bob Dylan</cd:artist> ");
		sb.append("<cd:country>USA</cd:country> ");
		sb.append("<cd:company>Columbia</cd:company> ");
		sb.append("<cd:price>10.90</cd:price> ");
		sb.append("<cd:year>1985</cd:year> ");
		sb.append("</rdf:Description> ");
		sb.append("<rdf:Description ");
		sb.append("rdf:about=\"http://www.recshop.fake/cd/Hideyourheart\"> ");
		sb.append("<cd:artist>Bonnie Tyler</cd:artist> ");
		sb.append("<cd:country>UK</cd:country> ");
		sb.append("<cd:company>CBS Records</cd:company> ");
		sb.append("<cd:price>9.90</cd:price> ");
		sb.append("<cd:year>1988</cd:year> ");
		sb.append("</rdf:Description> ");
		sb.append("</rdf:RDF>");
		rdfInvalidData = sb.toString().getBytes();
	}
	
	@BeforeClass
	public static void setUpBeforeClass() throws Exception {
		//String path;
		StringBuilder sb = new StringBuilder();

		//try {
			//JFig.initialize();
			//path = JFig.getInstance().getValue("mulgaraServer", "service");
			sb.append("select * from <");
			sb.append(JFig.getInstance().getValue("mulgaraServer", "GRAPH"));
			sb.append("> where {?s ?p ?o}");
			testquery = sb.toString();
		//} catch (JFigException e) {
		//	path = HttpManagement.SPARQL_SERVICE;
		//}
		//httpMan = new HttpManagement(path);
		httpMan = HttpManagement.getInstance();
	}
	
	@Ignore
	public void testHttpSparqlQueryOk() throws MulgaraException {
		String[] results = httpMan.httpSparqlQuery(testquery,
		"rdfxml");
		assertEquals(true, results != null);
	}
	//TODO Check this test, add boolean abortAll to test in case no connection
	@Ignore
	public void testHttpSparqlQueryNoconnection() {
		
		try {
			httpMan.httpSparqlQuery(testquery,
			"rdfxml");
		} catch (Exception e) {
			assertTrue(e instanceof MulgaraException);
		}
		
	}
	@SuppressWarnings("deprecation")
	@Test
	public void testHttpSparqlQueryWrongFormat() throws MulgaraException {

		String[] results = httpMan.httpSparqlQuery(testquery,
		"");
		assertEquals(null, results);
	}
	@SuppressWarnings("deprecation")
	@Test
	public void testHttpSparqlQueryEmptyQuery() throws MulgaraException {
		String [] results = httpMan.httpSparqlQuery("",
		"rdfxml");
		assertEquals(null, results);
	}
	@Test
	public void testHttpPostMulgaraRequestOk() throws MulgaraException {
		boolean res = httpMan.httpPostMulgaraRequest((String) DEMO_RDF);
		assertEquals(true, res);
	}
	@Test
	public void testHttpPostMulgaraRequestEmptyContents() {
		try {
			httpMan.httpPostMulgaraRequest(null);
		} catch (Exception e) {
			assertTrue(e instanceof MulgaraException);
		}
	}
	@Test
	public void testHttpPostMulgaraRequestEmptyGraph() {
		try {
			httpMan.setGraphURL("");
			httpMan.httpPostMulgaraRequest((String)DEMO_RDF);
		} catch (Exception e) {
			assertTrue(e instanceof MulgaraException);
		}
	}
	@Test
	public void testHttpPostMulgaraRequestInvalidContents() {
		try {
			httpMan.httpPostMulgaraRequest((String) new String(rdfInvalidData));
		} catch (Exception e) {
			assertTrue(e instanceof MulgaraException);
		}
	}
	
	@Test
	public void testHttpPostMulgaraRequestNoConnection() {
		try {
			httpMan.httpPostMulgaraRequest((String) DEMO_RDF);
		} catch (Exception e) {
			assertTrue(e instanceof MulgaraException);
		}
	}
}
