/*
 * Copyright 2013 Agustina Martinez-Garcia

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
 */
package uk.ljmu.mulgaraifz.tests;

import static org.junit.Assert.*;

import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;

//import org.igfay.jfig.JFig;
import org.igfay.jfig.JFigException;
import org.junit.BeforeClass;
import org.junit.Ignore;
import org.junit.Test;

import uk.ljmu.mulgaraifz.errorhandling.MulgaraException;
import uk.ljmu.mulgaraifz.httprequests.MulgaraOperations;
import uk.ljmu.mulgaraifz.httprequests.MulgaraOperationsImpl;
import uk.ljmu.mulgaraifz.model.DublinCore;

public class MulgaraOperationsImplTest {
	
	private static final String NS = "demo";
	private static final String testIdentifier = "demo:root";
	private static final String testIdentifierFull = "info:fedora/demo:root";
	private static MulgaraOperations mulgaraObj;
	//private static HttpManagement httpMan;
	
	@BeforeClass
	public static void setUpBeforeClass() throws Exception {
		
		//OJOOOO! Need to create Testing graph for Mulgara, just in case when running the tests that graph doesn't exist first or is empty
		//String basePath = JFig.getInstance().getValue("mulgaraServer", "service");
		//httpMan = new HttpManagement(basePath);
		//httpMan = HttpManagement.getInstance();
		mulgaraObj = new MulgaraOperationsImpl();
	}
	
	//TEST Environment
	@Ignore
	public void testInitializationErrors() {
		try {
			mulgaraObj = new MulgaraOperationsImpl();
		} catch (Exception e) {
			assertTrue(e instanceof MulgaraException);
		}
	}
	//TESTS getRootCollections
	@Test
	public void testGetRootCollectionsOk() throws MulgaraException {
		List<String> myList = mulgaraObj.getRootCollections();
		assertTrue(myList.size() >= 0);
	}
	//TESTS getRootCollectionsTitle
	@Test
	public void testGetRootCollectionsTitleOk() throws MulgaraException {
		List<String> myList = mulgaraObj.getRootCollectionsTitle();
		assertTrue(myList.size() >= 0);
	}

	//TESTS getCollectionInfo
	@Ignore
	public void testGetCollectionInfoOk() throws MulgaraException {
		DublinCore dcelement = mulgaraObj.getCollectionInfo(testIdentifier);
		assertNotNull(dcelement);
	}
	@Ignore
	public void testGetCollectionInfoFullId() throws MulgaraException {
		DublinCore dcelement = mulgaraObj.getCollectionInfo(testIdentifierFull);
		assertNotNull(dcelement);
	}
	@Test
	public void testGetCollectionInfoEmptyId() throws MulgaraException {
		DublinCore dcelement = mulgaraObj.getCollectionInfo("");
		assertNull(dcelement);
	}
	//TESTS getResourcesInfo
	@Test
	public void testGetResourcesInfoOk() throws MulgaraException {
		mulgaraObj.getResourcesInfo(testIdentifier);
		assertTrue(mulgaraObj.getResourceInfo().entrySet().size() >= 0);
	}
	
	@Test
	public void testGetResourcesInfoOkFullId() throws MulgaraException {
		mulgaraObj.getResourcesInfo(testIdentifierFull);
		assertTrue(mulgaraObj.getResourceInfo().entrySet().size() >= 0);
	}
	@Test
	public void testGetResourcesInfoOkEmptyId() {
		try {
			mulgaraObj.getResourcesInfo("");
		} catch (Exception e) {
			assertTrue(e instanceof MulgaraException);
		}
	}
	//TESTS getCollectionChildren
	@Test
	public void testGetCollectionChildrenOk() throws MulgaraException {
		
		List<String> myList = mulgaraObj.getCollectionChildren(testIdentifier);
		assertTrue(myList.size() >= 0);
	}
	
	@Test
	public void testGetCollectionChildrenFullId() throws MulgaraException {
		
		List<String> myList = mulgaraObj.getCollectionChildren(testIdentifierFull);
		assertTrue(myList.size() >= 0);
	}
	@Test
	public void testGetCollectionChildrenNoChildren() throws MulgaraException {
		
		List<String> myList = mulgaraObj.getCollectionChildren(testIdentifier);
		assertTrue(myList.size() >= 0);
	}
	
	@Test
	public void testGetCollectionChildrenEmptyId() {
		
		try {
			mulgaraObj.getCollectionChildren("");
		} catch (Exception e) {
			assertTrue(e instanceof MulgaraException);
		}
	}

	//TESTS getCollectionChildrenTitle
	@Test
	public void testGetCollectionChildrenTitleOk() throws MulgaraException {
		List<String> mList = mulgaraObj.getCollectionChildrenTitle(testIdentifier);
		assertTrue(mList.size() >= 0);
	}
	@Test
	public void testGetCollectionChildrenTitleEmptyId() {
		try {
			mulgaraObj.getCollectionChildrenTitle("");
		} catch (Exception e) {
			assertTrue(e instanceof MulgaraException);
		}
	}
	@Test
	public void testGetCollectionChildrenTitleFullId() throws MulgaraException {
		List<String> mList = mulgaraObj.getCollectionChildrenTitle(testIdentifierFull);
		assertTrue(mList.size() >= 0);
	}

	//TESTS getRecentCollections
	@Test
	public void testGetRecentCollectionsOk() throws MulgaraException {
		
		DateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss");
		Date date = new java.util.Date();
		//WEEK
		int MILLIS_IN_WEEK = 1000 * 60 * 60 * 24 * 7;
		String prevDate = dateFormat.format(date.getTime() - MILLIS_IN_WEEK);
		
		List<String> mList = mulgaraObj.getRecentCollections(prevDate);
		assertTrue(mList.size() >= 0);
	}
	
	@Test
	public void testGetRecentCollectionsEmptyDate() {
		try {
			mulgaraObj.getRecentCollections("");
		} catch (Exception e) {
			assertTrue(e instanceof MulgaraException);
		}
	}

	//TESTS getRecentCollectionsTitle
	@Test
	public void testGetRecentCollectionsTitleOk() throws MulgaraException {
		List<String> mList = mulgaraObj.getCollectionChildrenTitle(testIdentifier);
		assertTrue(mList.size() >= 0);
	}

	//TESTS getRecentResources
	@Test
	public void testGetRecentResourcesEmptyId() {
		try {
			mulgaraObj.getCollectionChildrenTitle("");
		} catch (Exception e) {
			assertTrue(e instanceof MulgaraException);
		}
	}

	//TESTS getObjectsTitles
	@Test
	public void testGetObjectsTitlesOk() throws MulgaraException {
		List<String> mList = mulgaraObj.getObjectsTitles(NS);
		assertTrue(mList.size() >= 0);
	}
	@Test
	public void testGetObjectsTitlesEmptyNs() {
		try {
			mulgaraObj.getObjectsTitles("");
		} catch (Exception e) {
			assertTrue(e instanceof MulgaraException);
		}
	}
	//TESTS getNumberOfChildrenTQL
	@Test
	public void testGetNumberOfChildrenTQLOk() throws MulgaraException, JFigException {
		String number = mulgaraObj.getNumberOfChildrenTQL(testIdentifier);
		assertTrue(!number.equals(""));
	}
	@Test
	public void testGetNumberOfChildrenTQLFullId() throws MulgaraException, JFigException {
		String number = mulgaraObj.getNumberOfChildrenTQL(testIdentifierFull);
		assertTrue(!number.equals(""));
	}
	@Test
	public void testGetNumberOfChildrenTQLEmptyId() {
		try {
			mulgaraObj.getNumberOfChildrenTQL("");
		} catch (Exception e) {
			assertTrue(e instanceof MulgaraException);
		}
	}
	@Ignore
	public void testGetNumberOfChildrenConfigError() {
		try {
			mulgaraObj.getNumberOfChildrenTQL("");
		} catch (Exception e) {
			assertTrue(e instanceof JFigException);
		}
	}

}
