/**
Copyright (c) 2010 Dennis Hotson

 Permission is hereby granted, free of charge, to any person
 obtaining a copy of this software and associated documentation
 files (the "Software"), to deal in the Software without
 restriction, including without limitation the rights to use,
 copy, modify, merge, publish, distribute, sublicense, and/or sell
 copies of the Software, and to permit persons to whom the
 Software is furnished to do so, subject to the following
 conditions:

 The above copyright notice and this permission notice shall be
 included in all copies or substantial portions of the Software.

 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 OTHER DEALINGS IN THE SOFTWARE.
*/

(function() {

jQuery.fn.springy = function(params) {
	var graph = this.graph = params.graph || new Graph();

	var stiffness = params.stiffness || 400.0;
	var repulsion = params.repulsion || 400.0;
	var damping = params.damping || 0.5;
	var nodeSelected = params.nodeSelected || null;

	var canvas = this[0];
	var ctx = canvas.getContext("2d");
	var imageCollection = new Image();
	imageCollection.src = './images/graph/new-folder.png';
	var imageFile = new Image();
	imageFile.src = './images/graph/File.png';
	var imageCategory = new Image();
	imageCategory.src = './images/graph/notes.png';
	var imagePDF = new Image();
	imagePDF.src = './images/graph/pdf.png';
	var imageVideo = new Image();
	imageVideo.src = './images/graph/video-icon.png';
	var imageText = new Image();
	imageText.src = './images/graph/text.png';
	var imageFlash = new Image();
	imageFlash.src = './images/graph/flash.png';
	var imageFlv = new Image();
	imageFlv.src = './images/graph/flv.png';
	var imageJP = new Image();
	imageJP.src = './images/graph/image.png';
	var imageWeb = new Image();
	imageWeb.src = './images/graph/html.png';

	var layout = this.layout = new Layout.ForceDirected(graph, stiffness, repulsion, damping);

	// calculate bounding box of graph layout.. with ease-in
	var currentBB = layout.getBoundingBox();
	var targetBB = {bottomleft: new Vector(-2, -2), topright: new Vector(2, 2)};

	// auto adjusting bounding box
	Layout.requestAnimationFrame(function adjust() {
		targetBB = layout.getBoundingBox();
		// current gets 20% closer to target every iteration
		currentBB = {
			bottomleft: currentBB.bottomleft.add( targetBB.bottomleft.subtract(currentBB.bottomleft)
				.divide(10)),
			topright: currentBB.topright.add( targetBB.topright.subtract(currentBB.topright)
				.divide(10))
		};

		Layout.requestAnimationFrame(adjust);
	});

	// convert to/from screen coordinates
	toScreen = function(p) {
		var size = currentBB.topright.subtract(currentBB.bottomleft);
		var sx = p.subtract(currentBB.bottomleft).divide(size.x).x * canvas.width;
		var sy = p.subtract(currentBB.bottomleft).divide(size.y).y * canvas.height;
		return new Vector(sx, sy);
	};

	fromScreen = function(s) {
		var size = currentBB.topright.subtract(currentBB.bottomleft);
		var px = (s.x / canvas.width) * size.x + currentBB.bottomleft.x;
		var py = (s.y / canvas.height) * size.y + currentBB.bottomleft.y;
		return new Vector(px, py);
	};

	// half-assed drag and drop
	var selected = null;
	var nearest = null;
	var dragged = null;

	jQuery(canvas).mousedown(function(e) {
		jQuery('.actions').hide();

		var pos = jQuery(this).offset();
		var p = fromScreen({x: e.pageX - pos.left, y: e.pageY - pos.top});
		selected = nearest = dragged = layout.nearest(p);

		if (selected.node !== null) {
			dragged.point.m = 10000.0;

			if (nodeSelected) {
				// AMG Added
				var type = '';
				nodeSelected(selected.node);
				var text = typeof(selected.node.data.label) !== 'undefined' ? selected.node.data.label : selected.node.id;
				if (text.indexOf('File') != -1) {
					type = 'File';
				} else if (text.indexOf('Category') != -1) {
					type = 'Cat';
				} else {
					type = 'Collection';
				}
				if (type === 'File') {
					var txt = document.getElementById("details");
					txt.innerHTML= "<h2>QuDEx Document</h2>" +
					"<div id=\"details-file\">" +
					"	<ul class=\"tabrow\">" +
					"		<li id=\"tab_1\" class=\"selected\"><a href=\'javascript:tabSwitch(\"tab_1\", \"content_1\");\'>General</a></li>" +
					"		<li id=\"tab_2\"><a href=\'javascript:tabSwitch(\"tab_2\", \"content_2\");\'>Item details</a></li>" +
					"		<li id=\"tab_3\"><a href=\'javascript:tabSwitch(\"tab_3\", \"content_3\");\'>Geographical Info</a></li>" +
					"	</ul>" +
					"	<div id=\"content_1\" style=\"display:block;\">" +
					"		<p><b>Title: </b>" + (typeof(selected.node.data.title) !== 'undefined' ? selected.node.data.title : 'N/A') + "</p>" +
					"		<p><b>Description: </b>" + (typeof(selected.node.data.desc) !== 'undefined' ? selected.node.data.desc : 'N/A') + "</p>" +
					"		<p><b>DC Type: </b>" + (typeof(selected.node.data.dc_type) !== 'undefined' ? selected.node.data.dc_type : 'N/A') + "</p>" +
					"	</div>" +
					"	<div id=\"content_2\" style=\"display:none;\">" +
					"		<p><b>Keywords: </b>" + (typeof(selected.node.data.subj) !== 'undefined' ? selected.node.data.subj : 'N/A') + "</p>" +
					"		<p><b>Coverage: </b>" + (typeof(selected.node.data.cov) !== 'undefined' ? selected.node.data.cov : 'N/A') + "</p>" +
					"		<p><b>Date: </b>" + (typeof(selected.node.data.date) !== 'undefined' ? selected.node.data.date : 'N/A') + "</p>" +
					"		<p><b>Format: </b>" + (typeof(selected.node.data.format) !== 'undefined' ? selected.node.data.format : 'N/A') + "</p>" +
					"	</div>" +
					"	<div id=\"content_3\" style=\"display:none;\">" +
					"		<p><b>Lat-long: </b>" + (typeof(selected.node.data.lat) !== 'undefined' ? selected.node.data.lat : 'N/A') + "</p>" +
					"		<p><b>Location: </b>" + (typeof(selected.node.data.loc) !== 'undefined' ? selected.node.data.loc : 'N/A') + "</p>" +					
					"	</div>" +
					"</div>";
				} else if (type === 'Collection') {
					var txt = document.getElementById("details");
					txt.innerHTML= "<h2>QuDEx Collection</h2>" + 
					"<div id=\"details-col\">" +
					"	<ul class=\"tabrow\">" +
					"		<li id=\"tab_1\" class=\"selected\"><a href=\'javascript:tabSwitch(\"tab_1\", \"content_1\");\'>General</a></li>" +
					"		<li id=\"tab_2\"><a href=\'javascript:tabSwitch(\"tab_2\", \"content_2\");\'>Item details</a></li>" +
					"		<li id=\"tab_3\"><a href=\'javascript:tabSwitch(\"tab_3\", \"content_3\");\'>Geographical Info</a></li>" +
					"		<li id=\"tab_4\"><a href=\'javascript:tabSwitch(\"tab_4\", \"content_4\");\'>Methodology Info</a></li>" +
					"	</ul>" +
					"	<div id=\"content_1\" style=\"display:block;\">" +
					"		<p><b>Title: </b>" + (typeof(selected.node.data.title) !== 'undefined' ? selected.node.data.title : 'N/A') + "</p>" +
					"		<p><b>Description: </b>" + (typeof(selected.node.data.desc) !== 'undefined' ? selected.node.data.desc : 'N/A') + "</p>" +
					"		<p><b>Type: </b>" + (typeof(selected.node.data.type) !== 'undefined' ? selected.node.data.type : 'N/A') + "</p>" +
					"	</div>" +
					"	<div id=\"content_2\" style=\"display:none;\">" +
					"		<p><b>Language: </b>" + (typeof(selected.node.data.lang) !== 'undefined' ? selected.node.data.lang : 'N/A') + "</p>" +
					"		<p><b>Keywords: </b>" + (typeof(selected.node.data.subj) !== 'undefined' ? selected.node.data.subj : 'N/A') + "</p>" +
					"		<p><b>Coverage: </b>" + (typeof(selected.node.data.cov) !== 'undefined' ? selected.node.data.cov : 'N/A') + "</p>" +
					"		<p><b>Rights: </b>" + (typeof(selected.node.data.rights) !== 'undefined' ? selected.node.data.rights : 'N/A') + "</p>" +
					"		<p><b>Contributor(s): </b>" + (typeof(selected.node.data.contr) !== 'undefined' ? selected.node.data.contr : 'N/A') + "</p>" +
					"		<p><b>Publisher: </b>" + (typeof(selected.node.data.pub) !== 'undefined' ? selected.node.data.pub : 'N/A') + "</p>" +
					"	</div>" +
					"	<div id=\"content_3\" style=\"display:none;\">" +
					"		<p><b>Lat-long: </b>" + (typeof(selected.node.data.lat) !== 'undefined' ? selected.node.data.lat : 'N/A') + "</p>" +
					"		<p><b>Location: </b>" + (typeof(selected.node.data.loc) !== 'undefined' ? selected.node.data.loc : 'N/A') + "</p>" +					
					"	</div>" +
					"	<div id=\"content_4\" style=\"display:none;\">" +
					"		<p><b>Sample: </b>" + (typeof(selected.node.data.univ) !== 'undefined' ? selected.node.data.univ : 'N/A') + "</p>" +
					"		<p><b>Sampling method: </b>" + (typeof(selected.node.data.samp) !== 'undefined' ? selected.node.data.samp : 'N/A') + "</p>" +
					"		<p><b>Time method: </b>" + (typeof(selected.node.data.time) !== 'undefined' ? selected.node.data.time : 'N/A') + "</p>" +
					"		<p><b>Data collector(s): </b>" + (typeof(selected.node.data.dataCol) !== 'undefined' ? selected.node.data.dataCol : 'N/A') + "</p>" +
					"		<p><b>Collection Mode: </b>" + (typeof(selected.node.data.colMode) !== 'undefined' ? selected.node.data.colMode : 'N/A') + "</p>" +
					"		<p><b>Additional Notes: </b>" + (typeof(selected.node.data.note) !== 'undefined' ? selected.node.data.note : 'N/A') + "</p>" +
					"	</div>" +
					"</div>";
				} else {
					var txt = document.getElementById("details");
					txt.innerHTML= "<h2>QuDEx Category</h2>" +
					"<div id=\"details-file\">" +
					"	<ul class=\"tabrow\">" +
					"		<li id=\"tab_1\" class=\"selected\"><a href=\'javascript:tabSwitch(\"tab_1\", \"content_1\");\'>General</a></li>" +
					"	</ul>" +
					"	<div id=\"content_1\" style=\"display:block;\">" +
					"		<p><b>Category label: </b>" + (typeof(selected.node.data.itemLabel) !== 'undefined' ? selected.node.data.itemLabel : 'N/A') + "</p>" +
					"		<p><b>Parent category: </b>" + ((typeof(selected.node.data.parentCat) !== 'undefined' && selected.node.data.parentCat != '') ? selected.node.data.parentCat : 'N/A') + "</p>" +
					"	</div>" +
					"</div>";
				}
			}
		}

		renderer.start();
	});

	jQuery(canvas).mousemove(function(e) {
		var pos = jQuery(this).offset();
		var p = fromScreen({x: e.pageX - pos.left, y: e.pageY - pos.top});
		nearest = layout.nearest(p);

		if (dragged !== null && dragged.node !== null) {
			dragged.point.p.x = p.x;
			dragged.point.p.y = p.y;
		}

		renderer.start();
	});

	jQuery(window).bind('mouseup',function(e) {
		dragged = null;
	});

	Node.prototype.getWidth = function() {
		var text = typeof(this.data.label) !== 'undefined' ? this.data.label : this.id;
		if (this._width && this._width[text])
			return this._width[text];

		ctx.save();
		ctx.font = "16px Verdana, sans-serif";
		var width = ctx.measureText(text).width + 10;
		ctx.restore();

		this._width || (this._width = {});
		this._width[text] = width;

		return width;
	};

	Node.prototype.getHeight = function() {
		return 20;
	};

	var renderer = new Renderer(1, layout,
		function clear() {
			ctx.clearRect(0,0,canvas.width,canvas.height);
		},
		function drawEdge(edge, p1, p2) {
			var x1 = toScreen(p1).x;
			var y1 = toScreen(p1).y;
			var x2 = toScreen(p2).x;
			var y2 = toScreen(p2).y;

			var direction = new Vector(x2-x1, y2-y1);
			var normal = direction.normal().normalise();

			var from = graph.getEdges(edge.source, edge.target);
			var to = graph.getEdges(edge.target, edge.source);

			var total = from.length + to.length;

			// Figure out edge's position in relation to other edges between the same nodes
			var n = 0;
			for (var i=0; i<from.length; i++) {
				if (from[i].id === edge.id) {
					n = i;
				}
			}

			var spacing = 6.0;

			// Figure out how far off center the line should be drawn
			var offset = normal.multiply(-((total - 1) * spacing)/2.0 + (n * spacing));

			var s1 = toScreen(p1).add(offset);
			var s2 = toScreen(p2).add(offset);

			var boxWidth = edge.target.getWidth();
			var boxHeight = edge.target.getHeight();

			var intersection = intersect_line_box(s1, s2, {x: x2-boxWidth/2.0, y: y2-boxHeight/2.0}, boxWidth, boxHeight);

			if (!intersection) {
				intersection = s2;
			}

			var stroke = typeof(edge.data.color) !== 'undefined' ? edge.data.color : '#000000';

			var arrowWidth;
			var arrowLength;

			var weight = typeof(edge.data.weight) !== 'undefined' ? edge.data.weight : 1.0;

			ctx.lineWidth = Math.max(weight *  2, 0.1);
			arrowWidth = 1 + ctx.lineWidth;
			arrowLength = 8;

			var directional = typeof(edge.data.directional) !== 'undefined' ? edge.data.directional : true;

			// line
			var lineEnd;
			if (directional) {
				lineEnd = intersection.subtract(direction.normalise().multiply(arrowLength * 0.5));
			} else {
				lineEnd = s2;
			}

			ctx.strokeStyle = stroke;
			ctx.beginPath();
			ctx.moveTo(s1.x, s1.y);
			ctx.lineTo(lineEnd.x, lineEnd.y);
			ctx.stroke();

			// arrow
			if (directional) {
				ctx.save();
				ctx.fillStyle = stroke;
				ctx.translate(intersection.x, intersection.y);
				ctx.rotate(Math.atan2(y2 - y1, x2 - x1));
				ctx.beginPath();
				ctx.moveTo(-arrowLength, arrowWidth);
				ctx.lineTo(0, 0);
				ctx.lineTo(-arrowLength, -arrowWidth);
				ctx.lineTo(-arrowLength * 0.8, -0);
				ctx.closePath();
				ctx.fill();
				ctx.restore();
			}

			// label
			if (typeof(edge.data.label) !== 'undefined') {
				text = edge.data.label;
				ctx.save();
				ctx.textAlign = "center";
				ctx.textBaseline = "top";
				ctx.font = "10px Helvetica, sans-serif";
				ctx.fillStyle = typeof(edge.data.colorLabel) !== 'undefined' ? edge.data.colorLabel : "#202530";
				ctx.fillText(text, (x1+x2)/2, (y1+y2)/2);
				ctx.restore();
			}

		},
		function drawNode(node, p) {
			var s = toScreen(p);
			var image;
			ctx.save();
			// AMG
			var text = typeof(node.data.label) !== 'undefined' ? node.data.label : node.id;
			if (text.indexOf('File') != -1) {
				switch (node.data.format) {
		            case "application/pdf":
		            	image = imagePDF;
		                break;
		            case "application/x-shockwave-flash":
		            	image = imageFlash;
		                break;
		            case "video/x-flv":
		            	image = imageFlv;
		                break;
		            case "video/quicktime":
		            	image = imageVideo;
		                break;
		            case "text/rtf":
		            	image = imageText;
		                break;
		            case "image/jpeg":
		            	image = imageJP;
		                break;
		            case "image/png":	
		            	image = imageJP;
		                break;
		            case "text/html":
		            	image = imageWeb;
		            	break;
		            default:
		                image = imageFile;
		                break;
		        }
				
				//type = 'File';
			} else if (text.indexOf('Category') != -1) {
				image = imageCategory;
				//type = 'Cat';
			} else {
				image = imageCollection;
				//type = 'Collection';
			}
			var boxWidth = image.width;
			var boxHeight = image.height;
			//var boxWidth = node.getWidth();
			//var boxHeight = node.getHeight();

			// clear background
			ctx.clearRect(s.x - boxWidth/2, s.y - 10, boxWidth, boxHeight);
			
			// fill background
			if (selected !== null && nearest.node !== null && selected.node.id === node.id) {
				//ctx.fillStyle = "#FFFFE0";
			} else if (nearest !== null && nearest.node !== null && nearest.node.id === node.id) {
				//ctx.fillStyle = "#EEEEEE";
			} else {
				//ctx.fillStyle = "#FFFFFF";
			}
			ctx.drawImage(image, s.x - boxWidth/2, s.y - 10, boxWidth, boxHeight);
			//ctx.fillRect(s.x - boxWidth/2, s.y - 10, boxWidth, 20);

			//ctx.textAlign = "left";
			//ctx.textBaseline = "top";
			//ctx.font = "16px Verdana, sans-serif";
			//ctx.fillStyle = "#000000";
			//ctx.font = "16px Verdana, sans-serif";
			//var text = typeof(node.data.label) !== 'undefined' ? node.data.label : node.id;
			//ctx.fillText(text, s.x - boxWidth/2 + 5, s.y - 8);

			ctx.restore();
		}
	);

	renderer.start();

	// helpers for figuring out where to draw arrows
	function intersect_line_line(p1, p2, p3, p4) {
		var denom = ((p4.y - p3.y)*(p2.x - p1.x) - (p4.x - p3.x)*(p2.y - p1.y));

		// lines are parallel
		if (denom === 0) {
			return false;
		}

		var ua = ((p4.x - p3.x)*(p1.y - p3.y) - (p4.y - p3.y)*(p1.x - p3.x)) / denom;
		var ub = ((p2.x - p1.x)*(p1.y - p3.y) - (p2.y - p1.y)*(p1.x - p3.x)) / denom;

		if (ua < 0 || ua > 1 || ub < 0 || ub > 1) {
			return false;
		}

		return new Vector(p1.x + ua * (p2.x - p1.x), p1.y + ua * (p2.y - p1.y));
	}

	function intersect_line_box(p1, p2, p3, w, h) {
		var tl = {x: p3.x, y: p3.y};
		var tr = {x: p3.x + w, y: p3.y};
		var bl = {x: p3.x, y: p3.y + h};
		var br = {x: p3.x + w, y: p3.y + h};

		var result;
		if (result = intersect_line_line(p1, p2, tl, tr)) { return result; } // top
		if (result = intersect_line_line(p1, p2, tr, br)) { return result; } // right
		if (result = intersect_line_line(p1, p2, br, bl)) { return result; } // bottom
		if (result = intersect_line_line(p1, p2, bl, tl)) { return result; } // left

		return false;
	}

	return this;
};

})();
/*var data = "data:image/svg+xml," +
		           "<svg xmlns='http://www.w3.org/2000/svg' width='200' height='200'>" +
		             "<foreignObject width='100%' height='100%'>" +
		               "<div xmlns='http://www.w3.org/1999/xhtml' style='font-size:12px'>" +
		                "<ul> <li style='color:red'> Title:" + node.data.title + 
		                "</li>  <li style='color:green'>thomas</li> </ul> "  +   
		               "</div>" +
		             "</foreignObject>" +
		           "</svg>";
				var img = new Image();
				img.src = data;
				img.onload = function() {
					ctx.drawImage(img, 25, 100);
					created = true;
				};
 * */
