/*
 * Copyright 2013 Agustina Martinez-Garcia

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
 */
package uk.ljmu.qudexapps.xmllib.tests;

import static org.junit.Assert.*;

import java.io.IOException;
import java.io.InputStream;

import org.junit.AfterClass;
import org.junit.BeforeClass; //import org.junit.Ignore;
import org.junit.Test;

import uk.ljmu.metadata.utilities.MetadataUtilities;
import uk.ljmu.qudexapps.xmllib.exceptions.XLSException;
import uk.ljmu.qudexapps.xmllib.services.ExcelManagementImpl;

public class ExcelManagementTest {
    private static InputStream isExcelFile;
    private static ExcelManagementImpl excelImp;

    @BeforeClass
    public static void setUpBeforeClass() throws Exception {
	ClassLoader classLoader = Thread.currentThread()
	.getContextClassLoader();

	excelImp = new ExcelManagementImpl();
	// AMG - Init the variables needed
	String fileSep = System.getProperty("file.separator", "/");
	StringBuilder filePath = new StringBuilder();
	filePath.append("test-spreadsheets/excel");
	filePath.append(fileSep);
	filePath.append("excel-general-ss.xls");
	isExcelFile = classLoader.getResourceAsStream(filePath.toString());
	filePath.delete(0, filePath.toString().length());
    }

    @AfterClass
    public static void setUpAfterClass() throws Exception {
	if (isExcelFile != null)
	    isExcelFile.close();
    }

    // Method readSpreadsheet
    @Test
    public void testReadSpreadsheetNullSS() {
	try {
	    excelImp.readSpreadsheet(null);
	} catch (XLSException e) {
	    System.out.println("Null spreadsheet to readSpreadsheet...");
	    assertTrue(true);
	} catch (IOException e) {
	    System.out.println("Null spreadsheet to readSpreadsheet...");
	    assertTrue(false);
	}
    }

    @Test
    public void testReadSpreadsheetEmptySS() {
	ClassLoader classLoader = Thread.currentThread()
	.getContextClassLoader();
	String fileSep = System.getProperty("file.separator", "/");
	StringBuilder filePath = new StringBuilder();
	filePath.append("test-spreadsheets/excel");
	filePath.append(fileSep);
	filePath.append("empty-ss.xls");
	InputStream is = classLoader.getResourceAsStream(filePath.toString());
	try {
	    excelImp.readSpreadsheet(is);
	    if (is != null)
		is.close();
	} catch (Exception e) {
	    System.out.println("Empty spreadsheet to readSpreadsheet...");
	    assertTrue(e instanceof XLSException);
	}
    }

    @Test
    public void testReadSpreadsheetInvalidFormatSS() {
	ClassLoader classLoader = Thread.currentThread()
	.getContextClassLoader();
	String fileSep = System.getProperty("file.separator", "/");
	StringBuilder filePath = new StringBuilder();
	filePath.append("test-spreadsheets/excel");
	filePath.append(fileSep);
	filePath.append("invalid-format.xlsx");
	InputStream is = classLoader.getResourceAsStream(filePath.toString());
	try {
	    excelImp.readSpreadsheet(is);
	    if (is != null)
		is.close();
	} catch (Exception e) {
	    System.out.println("Invalid format ss to readSpreadsheet...");
	    assertTrue(e instanceof XLSException);
	}
    }

    @Test
    public void testReadSpreadsheetOK() {
	try {
	    boolean res = excelImp.readSpreadsheet(isExcelFile);
	    assertTrue(res);
	} catch (XLSException e) {
	    System.out.println("Test failure, readSpreadsheet threw exception");
	    System.out.println(e.getMessage());
	    fail(e.getMessage());
	} catch (IOException e) {
	    System.out.println("Test failure, readSpreadsheet threw exception");
	    System.out.println(e.getMessage());
	    fail(e.getMessage());	 
	}
    }

    @Test
    public void testReadSpreadsheetWithBlanks() {
	ClassLoader classLoader = Thread.currentThread()
	.getContextClassLoader();
	String fileSep = System.getProperty("file.separator", "/");
	StringBuilder filePath = new StringBuilder();
	filePath.append("test-spreadsheets/excel");
	filePath.append(fileSep);
	filePath.append("excel-blanks-ss.xls");
	InputStream is = classLoader.getResourceAsStream(filePath.toString());
	try {
	    boolean res = excelImp.readSpreadsheet(is);
	    if (is != null)
		is.close();
	    assertTrue(res);
	} catch (XLSException e) {
	    System.out.println("Test failure, readSpreadsheet threw exception");
	    System.out.println(e.getMessage());
	    fail(e.getMessage());
	} catch (IOException e) {
	    System.out.println("Closing is");
	    fail(e.getMessage());
	}
    }

    // Method constructDocument
    @Test
    public void testConstructDocumentOKFoxml() {
	// Very important this test just applies to NON Qudex SS
	excelImp.setQudexSS(false);
	// Construct spreadsheet
	ClassLoader classLoader = Thread.currentThread()
	.getContextClassLoader();
	String fileSep = System.getProperty("file.separator", "/");
	StringBuilder filePath = new StringBuilder();
	filePath.append("test-spreadsheets/excel");
	filePath.append(fileSep);
	filePath.append("excel-general-ss.xls");
	InputStream is = classLoader.getResourceAsStream(filePath.toString());
	try {
	    boolean res = excelImp.readSpreadsheet(is);
	    if (is != null)
		is.close();
	    if (res) {
		String data = excelImp.constructDocument("demo:testexcel1",
			MetadataUtilities.ValueType.DcElement,
		"demo:testexcel1");
		assertTrue(!data.equals(""));
	    } else fail("Errors reading spreadsheet");
	} catch (XLSException e) {
	    System.out.println("Test failure, readSpreadsheet threw exception");
	    System.out.println(e.getMessage());
	    fail(e.getMessage());
	} catch (IOException e) {
	    System.out.println("Closing is");
	    fail(e.getMessage());
	}
    }

    @Test
    public void testConstructDocumentNoRelsDc() {
	// Construct spreadsheet
	ClassLoader classLoader = Thread.currentThread()
	.getContextClassLoader();
	String fileSep = System.getProperty("file.separator", "/");
	StringBuilder filePath = new StringBuilder();
	filePath.append("test-spreadsheets/excel");
	filePath.append(fileSep);
	filePath.append("excel-norelsdc-ss.xls");
	InputStream is = classLoader.getResourceAsStream(filePath.toString());
	try {
	    boolean res = excelImp.readSpreadsheet(is);
	    if (is != null)
		is.close();
	    if (res) {
		String data = excelImp.constructDocument("demo:test1",
			MetadataUtilities.ValueType.DcElement, "demo:test1");
		String data2 = excelImp.constructDocument("demo:test1",
			MetadataUtilities.ValueType.RELS_EXT, "demo:test1");
		assertTrue(data.equals("") && data2.equals(""));
	    } else fail("Errors while reading spreadsheet");
	} catch (XLSException e) {
	    System.out.println("Test failure, readSpreadsheet threw exception");
	    System.out.println(e.getMessage());
	    fail(e.getMessage());
	} catch (IOException e) {
	    System.out.println("Closing is");
	    fail(e.getMessage());
	}
    }

    @Test
    public void testConstructDocumentRDF() {
	// Construct spreadsheet
	ClassLoader classLoader = Thread.currentThread()
	.getContextClassLoader();
	String fileSep = System.getProperty("file.separator", "/");
	StringBuilder filePath = new StringBuilder();
	filePath.append("test-spreadsheets/excel");
	filePath.append(fileSep);
	filePath.append("excel-rdf-ss.xls");
	InputStream is = classLoader.getResourceAsStream(filePath.toString());
	try {
	    boolean res = excelImp.readSpreadsheet(is);
	    if (is != null)
		is.close();
	    if (res) {
		String data = excelImp.constructDocument("demo:test1",
			MetadataUtilities.ValueType.RELS_EXT, "demo:test1");
		assertTrue(!data.equals(""));
	    } else fail("Errors reading spreadsheet");
	} catch (XLSException e) {
	    System.out.println("Test failure, readSpreadsheet threw exception");
	    System.out.println(e.getMessage());
	    fail(e.getMessage());
	} catch (IOException e) {
	    System.out.println("Closing is");
	    fail(e.getMessage());
	}
    }
}
