/*
 * Copyright 2013 Agustina Martinez-Garcia

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
 */
package uk.ljmu.qudexapps.xmllib.tests;

import static org.junit.Assert.*;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.apache.xmlbeans.XmlError;
import org.apache.xmlbeans.XmlOptions;
import org.dom4j.Document;
import org.dom4j.DocumentException;
import org.dom4j.DocumentHelper;
import org.dom4j.Element;
import org.igfay.jfig.JFig;
import org.igfay.jfig.JFigException;
import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.Test;

import fedora.fedoraSystemDef.foxml.DatastreamType;
import fedora.fedoraSystemDef.foxml.StateType;
import fedora.fedoraSystemDef.foxml.DigitalObjectDocument.DigitalObject;

import uk.ljmu.metadata.schemasutilities.SchemasProcessing;
import uk.ljmu.metadata.utilities.MetadataUtilities;
import uk.ljmu.mulgaraifz.errorhandling.MulgaraException;
import uk.ljmu.qudexapps.xmllib.exceptions.FedoraException;
import uk.ljmu.qudexapps.xmllib.exceptions.FoxmlException;
import uk.ljmu.qudexapps.xmllib.exceptions.GraphException;
import uk.ljmu.qudexapps.xmllib.model.FoxmlBeans;
import uk.ljmu.qudexapps.xmllib.model.GraphInformation;
import uk.ljmu.qudexapps.xmllib.services.FedoraMulgaraServiceImpl;
import uk.ljmu.qudexapps.xmllib.services.QudexFoxmlService;
import uk.ljmu.qudexapps.xmllib.services.QudexFoxmlServiceImpl;
import uk.ljmu.qudexapps.xmllib.utilities.Constants;

public class FedoraMulgaraServiceImplTest {

    private static FedoraMulgaraServiceImpl fmService;
    private static boolean abortAll = false;
    private static String testCollectionPID = "qudex:collectiontest1";
    private static StringBuilder sbInline = new StringBuilder();
    private static QudexFoxmlService qudexServiceExcel;
    private static List<String> ingestedTestObjects;
    private static List<String> createdGraphs;
    static {
	sbInline
	.append("<rdf:RDF xmlns:rdf=\"http://www.w3.org/1999/02/22-rdf-syntax-ns#\" xmlns:ex=\"http://example.org/custom/\" xmlns:geo=\"http://www.w3.org/2003/01/geo/wgs84_pos#\">");
	sbInline
	.append("<rdf:Description rdf:about=\"info:fedora/qudex:collectiontest1\">");
	sbInline.append("<ex:label>Test custom rdf statements</ex:label>");
	sbInline.append("<ex:type>Project</ex:type>");
	sbInline
	.append("<ex:short_title>Changing Mathematics Teaching</ex:short_title>");
	sbInline
	.append("<ex:contributor_respondent>Easley, E.</ex:contributor_respondent>");
	sbInline.append("<geo:lat_long>39.774769,141.855469</geo:lat_long>");
	sbInline.append("</rdf:Description>");
	sbInline.append("</rdf:RDF>");
    };

    @BeforeClass
    public static void setUpBeforeClass() throws Exception {
	try {
	    fmService = new FedoraMulgaraServiceImpl();
	} catch (JFigException ex) {
	    abortAll = true;
	}
	qudexServiceExcel = new QudexFoxmlServiceImpl(
		MetadataUtilities.MIMETYPES_MAP.get("excel"), true);
	ingestedTestObjects = new ArrayList<String>();
	createdGraphs = new ArrayList<String>();
	createTestCollection();
    }

    @AfterClass
    public static void setUpAfterClass() throws Exception {
	removeTestObjects();
    }

    @Test
    public void testGetLoopIDbyCollection() {
	if (!abortAll) {
	    try {
		fmService.getLoopIDbyCollection(fmService.getMulgObject(), testCollectionPID);
	    } catch (JFigException e) {
		fail(e.getMessage());
	    } catch (MulgaraException e) {
		fail(e.getMessage());
	    } catch (GraphException e) {
		fail(e.getMessage());
	    }
	} else {
	    fail("Configration file not loaded properly...");
	}
    }
    
    @Test
    public void testCreate() {
	if (!abortAll) {
	    try {
		fmService.createGraphByCollection(testCollectionPID);
		Iterator<GraphInformation> it = fmService.getGraphInfo().iterator();
		if (it.hasNext()) {
		    GraphInformation gi = (GraphInformation) it.next();
		    createdGraphs.add(gi.getGraphName());
		} else fail("No graph generated");
	    } catch (JFigException e) {
		fail(e.getMessage());
	    } catch (MulgaraException e) {
		fail(e.getMessage());
	    } catch (GraphException e) {
		fail(e.getMessage());
	    } catch (FedoraException e) {
		fail(e.getMessage());
	    }
	} else {
	    fail("Configration file not loaded properly...");
	}
    }
    // PRIVATE METHODS
    private static boolean createTestCollection() throws JFigException {
	// AMG - Check that the collection doesn't exist
	if (qudexServiceExcel.getFedoraObject().getObjectXML(testCollectionPID).trim()
		.equals(testCollectionPID)) {
	    FoxmlBeans foxmlBeans = new FoxmlBeans(qudexServiceExcel
		    .getFedoraObject());
	    // Create object, it does not exist
	    StringBuilder sb = new StringBuilder();

	    boolean ok = createQudexFileTest(foxmlBeans, "qudex", "true");
	    if (ok) {
		// RELS_EXT RDF-XML CONTENTS FOR THE DS
		Document doc = null;

		// Construct RELS-EXT document
		List<String> prefixList = new ArrayList<String>();
		prefixList.add("rel");
		prefixList.add("rdf");
		prefixList.add("dcmitype");
		doc = SchemasProcessing.contructRDFDocument(foxmlBeans
			.getFoxmlDoc().getDigitalObject().getPID(), prefixList);

		String type = MetadataUtilities.DCMI_ELEMENTS.get("collection");
		SchemasProcessing.addRDFProperty("rdf", "rdf:type", type,
			MetadataUtilities.propertyTypes.Resource.toString(), doc);

		try {
		    foxmlBeans.addFedoraRelsExtDatastream(doc.asXML(),
			    Constants.DEFAULT_MIMETYPE);		    
		} catch (FoxmlException ex) {
		}

		// VALIDATE FOXML INSTANCE THROUGH SCHEMA
		try {
		    // CALL VALIDATION FUNCTION
		    // Create an XmlOptions instance and set the error listener.
		    XmlOptions validateOptions = new XmlOptions();
		    ArrayList<XmlError> errorList = new ArrayList<XmlError>();
		    validateOptions.setErrorListener(errorList);

		    // FOXML DOCUMENT VALIDATION
		    boolean validDocument = foxmlBeans.getFoxmlDoc().validate(
			    validateOptions);
		    // If the XML isn't valid, loop through the listener's
		    // contents,
		    // printing contained messages.
		    if (!validDocument) {
			for (int i = 0; i < errorList.size(); i++) {
			    XmlError error = (XmlError) errorList.get(i);
			    sb.append("RESULTS OF VALIDATION \nMessage: ");
			    sb.append(error.getMessage());
			    sb.append("Location of invalid XML: ");
			    sb.append(error.getCursorLocation().xmlText());
			    sb.delete(0, sb.toString().length());
			}
			System.out.println(sb.toString());
		    } else {
			// CALL TO INGEST METHOD
			String pid = qudexServiceExcel.getQuObject().ingestObjectFedora("Test Object",
				"qudex", foxmlBeans.getFoxmlDoc().toString(),
				foxmlBeans.getFoxmlDoc().getDigitalObject()
				.getPID());
			ingestedTestObjects.add(pid.trim());
			testCollectionPID = pid.trim();
		    }
		} catch (Exception e) {
		    System.out.println(e.getMessage());
		}
	    }
	    foxmlBeans.initFoxmlDocumentObject();
	    ok = createQudexFileTest(foxmlBeans, "qudex", "true");
	    if (ok) {
		// RELS_EXT RDF-XML CONTENTS FOR THE DS
		Document doc = null;

		// Construct RELS-EXT document
		List<String> prefixList = new ArrayList<String>();
		prefixList.add("rel");
		prefixList.add("rdf");
		prefixList.add("dcmitype");
		doc = SchemasProcessing.contructRDFDocument(foxmlBeans
			.getFoxmlDoc().getDigitalObject().getPID(), prefixList);

		SchemasProcessing.addRDFProperty("rel", "rel:isMemberOf", MetadataUtilities.OBJECTID + testCollectionPID,
			MetadataUtilities.propertyTypes.Resource.toString(), doc);
		SchemasProcessing.addRDFProperty("rel", "rel:isMemberOfCollection", MetadataUtilities.OBJECTID + testCollectionPID,
			MetadataUtilities.propertyTypes.Resource.toString(), doc);

		try {
		    foxmlBeans.addFedoraRelsExtDatastream(doc.asXML(),
			    Constants.DEFAULT_MIMETYPE);	
		    foxmlBeans.addDatastream("RDF_DS", DatastreamType.CONTROLGROUP.X, "RDF_DS datastream", StateType.A, sbInline.toString(), "text/xml", false, true);
		} catch (FoxmlException ex) {
		}

		// VALIDATE FOXML INSTANCE THROUGH SCHEMA
		try {
		    // CALL VALIDATION FUNCTION
		    // Create an XmlOptions instance and set the error listener.
		    XmlOptions validateOptions = new XmlOptions();
		    ArrayList<XmlError> errorList = new ArrayList<XmlError>();
		    validateOptions.setErrorListener(errorList);

		    // FOXML DOCUMENT VALIDATION
		    boolean validDocument = foxmlBeans.getFoxmlDoc().validate(
			    validateOptions);
		    // If the XML isn't valid, loop through the listener's
		    // contents,
		    // printing contained messages.
		    if (!validDocument) {
			for (int i = 0; i < errorList.size(); i++) {
			    XmlError error = (XmlError) errorList.get(i);
			    sb.append("RESULTS OF VALIDATION \nMessage: ");
			    sb.append(error.getMessage());
			    sb.append("Location of invalid XML: ");
			    sb.append(error.getCursorLocation().xmlText());
			    sb.delete(0, sb.toString().length());
			}
			System.out.println(sb.toString());
		    } else {
			// CALL TO INGEST METHOD
			String pid = qudexServiceExcel.getQuObject().ingestObjectFedora("Test Objects",
				"qudex", foxmlBeans.getFoxmlDoc().toString(),
				foxmlBeans.getFoxmlDoc().getDigitalObject()
				.getPID());
			ingestedTestObjects.add(pid.trim());
		    }
		} catch (Exception e) {
		    System.out.println(e.getMessage());
		}
	    }
	    //Add File Member
	}
	return true;
    }
    
    private static boolean createQudexFileTest(FoxmlBeans foxmlBeans,
	    String namespace, String xmlFormat) throws JFigException {
	String pid = "";
	String pidListXML = "";
	DigitalObject digObj;
	// AMG- Obtain current Foxml Digital Object instance
	digObj = foxmlBeans.getFoxmlDoc().getDigitalObject();
	foxmlBeans
	.addLabelProperty("Test qudex collection, FedoraMulgaraService tests.");
	foxmlBeans.addOwnerIDProperty(JFig.getInstance().getValue("fedoraServer", "user"));
	foxmlBeans.addStateProperty("A");
	// AMg- Parameters: number of PIDs, optional namespace associate to
	// the
	// PID, XML output format
	pidListXML = qudexServiceExcel.getFedoraObject().getNextPID(1, namespace,
		xmlFormat);

	if (pidListXML == null || pidListXML == "") {
	    return false;
	}

	// AMG- Parse XML response to obtain the generated PID
	try {
	    Document doc = DocumentHelper.parseText(pidListXML);
	    Element pidElem = doc.getRootElement().element("pid");
	    pid = pidElem.getText();
	    digObj.setPID(pid);
	} catch (DocumentException ex) {
	    return false;
	}

	return true;
    }
    private static void removeTestObjects() {
	for (String pid : ingestedTestObjects) {
	    qudexServiceExcel.getFedoraObject().deleteObject(pid);
	}
	for (String graphUri : createdGraphs) {
	    fmService.dropGraph(graphUri);
	}
    }
}
