/*
 * Copyright 2013 Agustina Martinez-Garcia

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
 */
package uk.ljmu.qudexapps.xmllib.tests;

import static org.junit.Assert.*;

import org.junit.BeforeClass;
import org.junit.Test;

import fedora.fedoraSystemDef.foxml.DatastreamType;
import fedora.fedoraSystemDef.foxml.StateType;

import uk.ljmu.qudexapps.xmllib.exceptions.FoxmlException;
import uk.ljmu.qudexapps.xmllib.model.FoxmlBeans;

public class FoxmlBeansTest {
    private static FoxmlBeans foxmlBeans;
    private static StringBuilder sbInline = new StringBuilder();
    private static StringBuilder sbRelsExt = new StringBuilder();
    private static StringBuilder sbDC = new StringBuilder();

    static {
	sbDC
	.append("<oai_dc:dc xmlns:oai_dc=\"http://www.openarchives.org/OAI/2.0/oai_dc/\" xmlns:dc=\"http://purl.org/dc/elements/1.1/\" xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" xsi:schemaLocation=\"http://www.openarchives.org/OAI/2.0/oai_dc/ http://www.openarchives.org/OAI/2.0/oai_dc.xsd\">");
	sbDC.append("  <dc:title>Test qudex collection</dc:title>");
	sbDC.append("  <dc:creator>AMG</dc:creator>");
	sbDC.append("  <dc:subject>qualitative data</dc:subject>");
	sbDC
	.append("  <dc:description>Qudex collection description</dc:description>");
	sbDC.append("  <dc:publisher>AMG</dc:publisher>");
	sbDC.append("  <dc:contributor>AMG</dc:contributor>");
	sbDC.append("  <dc:date>2010-12-01</dc:date>");
	sbDC.append("  <dc:type>Collection</dc:type>");
	sbDC.append("  <dc:identifier>demo:test1</dc:identifier>");
	sbDC.append("  <dc:language>EN</dc:language>");
	sbDC.append("  <dc:coverage>UK</dc:coverage>");
	sbDC.append("  <dc:rights>AMG</dc:rights>");
	sbDC.append("</oai_dc:dc>");

	sbRelsExt
	.append("<rdf:RDF xmlns:rdf=\"http://www.w3.org/1999/02/22-rdf-syntax-ns#\" xmlns:geo=\"http://www.w3.org/2003/01/geo/wgs84_pos#\">");
	sbRelsExt
	.append("  <rdf:Description rdf:about=\"info:fedora/demo:test1\">");
	sbRelsExt
	.append("    <rdf:type rdf:resource=\"http://purl.org/dc/dcmitype/Collection\"></rdf:type>");
	sbRelsExt.append("    <geo:location>UK</geo:location>");
	sbRelsExt.append("    <geo:lat_long>53.4, -3</geo:lat_long>");
	sbRelsExt.append("  </rdf:Description>");
	sbRelsExt.append("</rdf:RDF>");

	sbInline
	.append("<rdf:RDF xmlns:rdf=\"http://www.w3.org/1999/02/22-rdf-syntax-ns#\" xmlns:ex=\"http://example.org/custom/\" xmlns:geo=\"http://www.w3.org/2003/01/geo/wgs84_pos#\">");
	sbInline
	.append("  <rdf:Description rdf:about=\"info:fedora/demo:test1\">");
	sbInline
	.append("    <ex:label>info:fedora/edeval:projects8</ex:label>");
	sbInline.append("    <ex:type>Project</ex:type>");
	sbInline
	.append("    <ex:short_title>Changing Mathematics Teaching</ex:short_title>");
	sbInline
	.append("    <ex:contributor_respondent>Easley, E.</ex:contributor_respondent>");
	sbInline
	.append("    <geo:lat_long>39.774769,141.855469</geo:lat_long>");
	sbInline.append("  </rdf:Description>");
	sbInline.append("</rdf:RDF>");

    }

    @BeforeClass
    public static void setUpBeforeClass() throws Exception {
	foxmlBeans = new FoxmlBeans();
    }

    // Method addDatastream
    @Test
    public void testAddDatastreamOK() {
	// Set test object properties
	foxmlBeans.addLabelProperty("Test");
	foxmlBeans.addStateProperty("A");
	foxmlBeans.addOwnerIDProperty("fedoraAdmin");
	try {
	    foxmlBeans.addDatastream("DATA", DatastreamType.CONTROLGROUP.X,
		    "Test DS creation", StateType.A, sbInline.toString(),
		    "text/xml", false, false);
	    boolean valid = foxmlBeans.getFoxmlDoc().getDigitalObject()
	    .validate();
	    assertTrue(valid);
	} catch (FoxmlException e) {
	    fail(e.getMessage());
	}
    }

    @Test
    public void testAddDatastreamInvalidDsType() {
	// Set test object properties
	foxmlBeans.addLabelProperty("Test");
	foxmlBeans.addStateProperty("A");
	foxmlBeans.addOwnerIDProperty("fedoraAdmin");
	try {
	    foxmlBeans.addDatastream("DATA", DatastreamType.CONTROLGROUP.M,
		    "Test DS creation", StateType.A, sbInline.toString(),
		    "text/xml", false, false);
	    boolean valid = foxmlBeans.getFoxmlDoc().getDigitalObject()
	    .validate();
	    assertFalse(valid);
	} catch (Exception e) {
	    assertTrue(e instanceof FoxmlException);
	}
    }
    // TODO FINISH TESTS
}
