/*
 * Copyright 2013 Agustina Martinez-Garcia

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
 */
package uk.ljmu.qudexapps.manager.controllers;

import java.io.IOException;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.servlet.ModelAndView;
import org.springframework.webflow.core.FlowException;
import org.springframework.webflow.execution.FlowExecutionOutcome;
import org.springframework.webflow.execution.repository.NoSuchFlowExecutionException;
import org.springframework.webflow.mvc.servlet.AbstractFlowHandler;

import uk.ljmu.mulgaraifz.errorhandling.MulgaraException;
import uk.ljmu.qudexapps.xmllib.exceptions.FoxmlException;

public class ProcessFlowController extends AbstractFlowHandler {
    @Override
    public String handleExecutionOutcome(FlowExecutionOutcome outcome,
	    HttpServletRequest request, HttpServletResponse response) {
	return "intro";
    }

    @Override
    public String handleException(FlowException e, HttpServletRequest request,
	    HttpServletResponse response) {
	if (e instanceof NoSuchFlowExecutionException) {
	    return "intro";
	} else {
	    return "error/defaultError";
	}
    }

    @ExceptionHandler(IOException.class)
    public String handleIOException(IOException ex, HttpServletRequest request) {
	return "error/defaultError";
    }

    @ExceptionHandler(FoxmlException.class)
    public ModelAndView handleFoxmlException(FoxmlException ex,
	    HttpServletRequest request) {
	ModelAndView mav = new ModelAndView("error/foxmlError");
	mav.addObject("errorString", ex.getMessage());
	return mav;
    }

    @ExceptionHandler(MulgaraException.class)
    public ModelAndView handleMulgaraException(MulgaraException ex,
	    HttpServletRequest request) {
	ModelAndView mav = new ModelAndView("error/mulgaraError");
	mav.addObject("errorString", ex.getMessage());
	return mav;
    }
}
