/*
 * Copyright 2013 Agustina Martinez-Garcia

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
 */
package uk.ljmu.qudexapps.manager.validators;

import org.springframework.validation.Errors;
import org.springframework.validation.Validator;

import uk.ljmu.qudexapps.manager.domain.FilesModel;

public class FilesValidator implements Validator {

    public boolean supports(Class<?> arg0) {
	return FilesModel.class.equals(arg0);
    }

    public void validate(Object files, Errors errors) {
	FilesModel myFiles = (FilesModel) files;

	if (myFiles.getSpreadsheet().getOriginalFilename() == "") {
	    errors.rejectValue("spreadsheet", "noFilesSelected");
	}
	if (myFiles.getSpreadsheet().getOriginalFilename() != "") {

	    if (!myFiles.getSpreadsheet().getContentType().equals(
		    "application/vnd.ms-excel")
		    && !myFiles.getSpreadsheet().getContentType().equals(
			    "text/csv")
		    && !myFiles.getSpreadsheet().getContentType().equals(
			    "application/ms-excel")
		    && !myFiles.getSpreadsheet().getContentType().equals(
			    "application/x-msexcel")) {
		errors.rejectValue("fedoraFile", "invalidFedoraFile");
	    }
	}
	 String[] collectionsArray = myFiles.getCollectionsIds().split(";");
	 //Check that only one collection ID is provided
	 if (collectionsArray.length > 1) {
	    //Error
	    errors.rejectValue("collectionsIds", "multipleIDS");
	}
    }

}
