/*
 * Copyright 2013 Agustina Martinez-Garcia

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
 */
package uk.ljmu.qudexapps.swfspreadsheet.actions;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.StringWriter;
import java.io.Writer;
import java.util.Iterator;

import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;

import org.apache.log4j.Logger;
import org.dom4j.Document;
import org.dom4j.DocumentException;
import org.dom4j.DocumentHelper;
import org.dom4j.Element;
import org.springframework.webflow.action.AbstractAction;
import org.springframework.webflow.execution.Event;
import org.springframework.webflow.execution.RequestContext;

import uk.ljmu.qudexapps.swfspreadsheet.domain.DdiObject;
import uk.ljmu.qudexapps.swfspreadsheet.domain.Files;

public class ParseDdiAction extends AbstractAction{

    private static Logger log = Logger.getLogger(ParseDdiAction.class);

    @Override
    protected Event doExecute(RequestContext context) throws Exception {
	Files myFiles = (Files) context.getFlowScope().get("files");
	this.parseDDI(myFiles);
	return success();
    }
    
    private boolean parseDDI(Files files) {
	//Class loader to load the XSLT file
	ClassLoader classLoader = Thread.currentThread()
	.getContextClassLoader();
	InputStream is = classLoader.getResourceAsStream("ddi-transform.xslt");
	Source xslSource = null;
	try {
	    TransformerFactory  tFactory =
	        TransformerFactory.newInstance();
	    xslSource = new StreamSource(is);
	    Transformer transformer = tFactory.newTransformer(xslSource);
	    Writer outWriter = new StringWriter();  
	    StreamResult result = new StreamResult( outWriter );
	    String fullPath = files.getTempDirPath() + System.getProperty("file.separator") + files.getFile().getOriginalFilename();
	    FileInputStream fileIs = new FileInputStream(new File(fullPath));
	    transformer.transform(new StreamSource(fileIs), result);
	    StringWriter sw = (StringWriter) result.getWriter();
	    //System.out.println(sw);
	    // otherwise you'll get a class cast exception here
	    StringBuffer sb = sw.getBuffer(); 
	    Document doc = DocumentHelper.parseText(sb.toString());
	    Element root = doc.getRootElement();
	    Iterator<?> it = root.elementIterator("field");
	    String sampProc = "", universe = "", timeM = "", dataC = "", collMode = "", desc = "";
	    
	    while (it.hasNext()) {
		Element elem = (Element) it.next();
		// samplingProcedure;
		
		if (elem.attribute("name").getStringValue().equalsIgnoreCase("samplingProcedure")) {
		    sampProc += elem.getText() + ";"; 
		} else if (elem.attribute("name").getStringValue().equalsIgnoreCase("universe")) {
		    universe += elem.getText() + ";";
		} else if (elem.attribute("name").getStringValue().equalsIgnoreCase("timeMethod")) {
		    timeM += elem.getText() + ";";
		} else if (elem.attribute("name").getStringValue().equalsIgnoreCase("dataCollector")) {
		    dataC += elem.getText() + ";";
		} else if (elem.attribute("name").getStringValue().equalsIgnoreCase("collectionMode")) {
		    collMode += elem.getText() + ";";
		} else if (elem.attribute("name").getStringValue().equalsIgnoreCase("description") && desc == "") {
		    desc = elem.getText();
		}   
	    }
	    DdiObject ddiObj = new DdiObject();
	    
	    ddiObj.setCollectionMode(collMode);
	    ddiObj.setSamplingProcedure(sampProc);
	    ddiObj.setTimeMethod(timeM);
	    ddiObj.setUniverse(universe);
	    ddiObj.setDataCollector(dataC);
	    ddiObj.setDescription(desc);
	    files.getDdiObjectList().add(ddiObj);
	    
	} catch (TransformerConfigurationException e) {
	    log.error("Error: ", e);
	} catch (TransformerException e) {
	    log.error("Error: ", e);
	} catch (DocumentException e) {
	    log.error("Error: ", e);
	} catch (IOException e) {
	    log.error("Error: ", e);
	} finally {
	    if (is != null)
			try {
				is.close();
			} catch (IOException e) {
				// TODO Auto-generated catch block
				log.error("Error: ", e);
			}
	}
	return true;
    }

}
